#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_utils import timeutils

from zun.api import servicegroup
from zun.common import context
from zun.common import exception
from zun import objects
from zun.scheduler import filter_scheduler
from zun.tests import base
from zun.tests.unit.db import utils
from zun.tests.unit.scheduler.fakes import FakeService


class FilterSchedulerTestCase(base.TestCase):
    """Test case for Filter Scheduler."""

    driver_cls = filter_scheduler.FilterScheduler

    def setUp(self):
        super(FilterSchedulerTestCase, self).setUp()
        self.mock_placement_client = mock.Mock()
        p = mock.patch('zun.scheduler.client.report.SchedulerReportClient',
                       return_value=self.mock_placement_client)
        p.start()
        self.addCleanup(p.stop)
        self.context = context.RequestContext('fake_user', 'fake_project')
        self.driver = self.driver_cls()

    @mock.patch.object(servicegroup.ServiceGroup, 'service_is_up')
    @mock.patch.object(objects.ComputeNode, 'list')
    @mock.patch.object(objects.ZunService, 'list_by_binary')
    @mock.patch('random.choice')
    def test_select_destinations(self, mock_random_choice,
                                 mock_list_by_binary, mock_compute_list,
                                 mock_service_is_up):
        all_services = [FakeService('service1', 'host1'),
                        FakeService('service2', 'host2'),
                        FakeService('service3', 'host3'),
                        FakeService('service4', 'host4')]

        def _return_services(*args, **kwargs):
            return all_services

        self.driver.servicegroup_api.service_is_up = mock.Mock(
            return_value=True)
        mock_list_by_binary.side_effect = _return_services
        numa_topology = {
            "nodes": [
                {
                    "id": 0,
                    "cpuset": [1, 2, 3, 4],
                    "pinned_cpus": [],
                    "mem_total": 1024 * 64,
                    "mem_available": 1024 * 64
                },
                {
                    "id": 1,
                    "cpuset": [5, 6, 7, 8],
                    "pinned_cpus": [],
                    "mem_total": 1024 * 64,
                    "mem_available": 1024 * 64
                }
            ]
        }
        numa = objects.numa.NUMATopology._from_dict(numa_topology)
        test_container = utils.get_test_container()
        containers = [objects.Container(self.context, **test_container)]
        node1 = objects.ComputeNode(self.context)
        node1.rp_uuid = mock.sentinel.node1_rp_uuid
        node1.updated_at = timeutils.utcnow()
        node1.cpus = 48
        node1.cpu_used = 0.0
        node1.mem_total = 1024 * 128
        node1.mem_used = 1024 * 4
        node1.mem_free = 1024 * 124
        node1.mem_available = 1024 * 124
        node1.disk_total = 80
        node1.disk_used = 20
        node1.hostname = 'host1'
        node1.numa_topology = numa
        node1.labels = {}
        node1.pci_device_pools = None
        node1.disk_quota_supported = True
        node1.runtimes = ['runc']
        node1.enable_cpu_pinning = False
        node2 = objects.ComputeNode(self.context)
        node2.rp_uuid = mock.sentinel.node2_rp_uuid
        node2.updated_at = timeutils.utcnow()
        node2.cpus = 48
        node2.cpu_used = 0.0
        node2.mem_total = 1024 * 128
        node2.mem_used = 1024 * 4
        node2.mem_free = 1024 * 124
        node2.mem_available = 1024 * 124
        node2.disk_total = 80
        node2.disk_used = 20
        node2.hostname = 'host2'
        node2.numa_topology = numa
        node2.labels = {}
        node2.pci_device_pools = None
        node2.disk_quota_supported = True
        node2.runtimes = ['runc']
        node2.enable_cpu_pinning = False
        node3 = objects.ComputeNode(self.context)
        node3.rp_uuid = mock.sentinel.node3_rp_uuid
        node3.updated_at = timeutils.utcnow()
        node3.cpus = 48
        node3.cpu_used = 0.0
        node3.mem_total = 1024 * 128
        node3.mem_used = 1024 * 4
        node3.mem_free = 1024 * 124
        node3.mem_available = 1024 * 124
        node3.disk_total = 80
        node3.disk_used = 20
        node3.hostname = 'host3'
        node3.numa_topology = numa
        node3.labels = {}
        node3.pci_device_pools = None
        node3.disk_quota_supported = True
        node3.runtimes = ['runc']
        node3.enable_cpu_pinning = False
        node4 = objects.ComputeNode(self.context)
        node4.rp_uuid = mock.sentinel.node4_rp_uuid
        node4.updated_at = timeutils.utcnow()
        node4.cpus = 48
        node4.cpu_used = 0.0
        node4.mem_total = 1024 * 128
        node4.mem_used = 1024 * 4
        node4.mem_free = 1024 * 124
        node4.mem_available = 1024 * 124
        node4.disk_total = 80
        node4.disk_used = 20
        node4.hostname = 'host4'
        node4.numa_topology = numa
        node4.labels = {}
        node4.pci_device_pools = None
        node4.disk_quota_supported = True
        node4.runtimes = ['runc']
        node4.enable_cpu_pinning = False
        nodes = [node1, node2, node3, node4]
        mock_compute_list.return_value = nodes

        mock_service_is_up.return_value = True
        extra_spec = {}
        mock_alloc_reqs_by_rp_uuid = {
            node3.rp_uuid: [mock.sentinel.node3_alloc_req]
        }
        mock_provider_summaries = {
            node3.rp_uuid: {}
        }

        dests = self.driver.select_destinations(
            self.context, containers, extra_spec,
            mock_alloc_reqs_by_rp_uuid, mock_provider_summaries,
            mock.sentinel.alloc_request_version)

        self.assertEqual(1, len(dests))
        (host, node) = (dests[0]['host'], dests[0]['nodename'])
        self.assertEqual('host3', host)
        self.assertIsNone(node)
        container = containers[0]
        self.mock_placement_client.claim_resources.assert_called_once_with(
            mock.ANY, container.uuid, mock.sentinel.node3_alloc_req,
            container.project_id, container.user_id,
            allocation_request_version=mock.sentinel.alloc_request_version,
            consumer_generation=None)

    @mock.patch.object(objects.ComputeNode, 'list')
    @mock.patch.object(objects.ZunService, 'list_by_binary')
    @mock.patch('random.choice')
    def test_select_destinations_no_valid_host(self, mock_random_choice,
                                               mock_list_by_binary,
                                               mock_compute_list):

        def _return_services(*args, **kwargs):
            return []

        self.driver.servicegroup_api.service_is_up = mock.Mock(
            return_value=True)
        mock_list_by_binary.side_effect = _return_services
        test_container = utils.get_test_container()
        containers = [objects.Container(self.context, **test_container)]
        extra_spec = {}
        mock_alloc_reqs_by_rp_uuid = {}
        mock_provider_summaries = {}
        self.assertRaises(exception.NoValidHost,
                          self.driver.select_destinations, self.context,
                          containers, extra_spec, mock_alloc_reqs_by_rp_uuid,
                          mock_provider_summaries,
                          mock.sentinel.alloc_request_version)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_policy import policy

from zun.common.policies import base

CONTAINER = 'container:%s'

rules = [
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'create',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Create a new container.',
        operations=[
            {
                'path': '/v1/containers',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'create:runtime',
        check_str=base.RULE_ADMIN_API,
        description='Create a new container with specified runtime.',
        operations=[
            {
                'path': '/v1/containers',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'create:privileged',
        check_str=base.RULE_DENY_EVERYBODY,
        description=('Create a new privileged container.'
                     'Warning: the privileged container has a big security '
                     'risk so be caution if you want to enable this feature'),
        operations=[
            {
                'path': '/v1/containers',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'create:requested_destination',
        check_str=base.RULE_ADMIN_API,
        description=('Create a container on the requested compute host.'),
        operations=[
            {
                'path': '/v1/containers',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'create:image_pull_policy',
        check_str=base.RULE_ADMIN_API,
        description=('Create a new container with specified image pull '
                     'policy.'),
        operations=[
            {
                'path': '/v1/containers',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'delete',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Delete a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'DELETE'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'delete_all_projects',
        check_str=base.RULE_ADMIN_API,
        description='Delete a container from all projects.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'DELETE'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'delete_force',
        check_str=base.RULE_ADMIN_API,
        description='Forcibly delete a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'DELETE'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'get_one',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Retrieve the details of a specific container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'get_one:host',
        check_str=base.RULE_ADMIN_API,
        description='Retrieve the host field of containers.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'GET'
            },
            {
                'path': '/v1/containers',
                'method': 'GET'
            },
            {
                'path': '/v1/containers',
                'method': 'POST'
            },
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'PATCH'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'get_one:image_pull_policy',
        check_str=base.RULE_ADMIN_API,
        description='Retrieve the image_pull_policy field of containers.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'GET'
            },
            {
                'path': '/v1/containers',
                'method': 'GET'
            },
            {
                'path': '/v1/containers',
                'method': 'POST'
            },
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'PATCH'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'get_one:privileged',
        check_str=base.RULE_ADMIN_API,
        description='Retrieve the privileged field of containers.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'GET'
            },
            {
                'path': '/v1/containers',
                'method': 'GET'
            },
            {
                'path': '/v1/containers',
                'method': 'POST'
            },
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'PATCH'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'get_one:runtime',
        check_str=base.RULE_ADMIN_API,
        description='Retrieve the runtime field of containers.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'GET'
            },
            {
                'path': '/v1/containers',
                'method': 'GET'
            },
            {
                'path': '/v1/containers',
                'method': 'POST'
            },
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'PATCH'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'get_one_all_projects',
        check_str=base.RULE_ADMIN_API,
        description=('Retrieve the details of a specific container from '
                     'all projects.'),
        operations=[
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'get_all',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Retrieve the details of all containers.',
        operations=[
            {
                'path': '/v1/containers',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'get_all_all_projects',
        check_str=base.RULE_ADMIN_API,
        description='Retrieve the details of all containers across projects.',
        operations=[
            {
                'path': '/v1/containers',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'update',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Update a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}',
                'method': 'PATCH'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'start',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Start a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/start',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'stop',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Stop a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/stop',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'reboot',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Reboot a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/reboot',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'pause',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Pause a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/pause',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'unpause',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Unpause a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/unpause',
                'method': 'POST'
            }
        ]
    ),
    # FIXME(lbragstad): This API call isn't actually listed in zun's API
    # reference (bug #1720924):
    # https://docs.openstack.org/api-ref/application-container/
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'logs',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Get the log of a container',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/logs',
                'method': 'GET'
            }
        ]
    ),
    # FIXME(lbragstad): This API call isn't actually listed in zun's API
    # reference (bug #1720925):
    # https://docs.openstack.org/api-ref/application-container/
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'execute',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Execute command in a running container',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/execute',
                'method': 'POST'
            }
        ]
    ),
    # FIXME(lbragstad): This API call isn't actually listed in zun's API
    # reference (bug #1720926):
    # https://docs.openstack.org/api-ref/application-container/
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'execute_resize',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Resize the TTY used by an execute command.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/execute_resize',
                'method': 'POST'
            }
        ]
    ),
    # FIXME(lbragstad): This API call isn't actually listed in zun's API
    # reference (bug #1720927):
    # https://docs.openstack.org/api-ref/application-container/
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'kill',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Kill a running container',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/kill',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'rename',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Rename a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/rename',
                'method': 'POST'
            }
        ]
    ),
    # FIXME(lbragstad): This API call isn't actually listed in zun's API
    # reference (bug #1720928):
    # https://docs.openstack.org/api-ref/application-container/
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'attach',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Attach to a running container',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/attach',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'resize',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Resize a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/resize',
                'method': 'POST'
            }
        ]
    ),
    # FIXME(lbragstad): This API call isn't actually listed in zun's API
    # reference (bug #1720929):
    # https://docs.openstack.org/api-ref/application-container/
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'top',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Display the running processes inside the container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/top',
                'method': 'GET'
            }
        ]
    ),
    # FIXME(lbragstad): This API call isn't actually listed in zun's API
    # reference, verify with someone from zun:
    # https://docs.openstack.org/api-ref/application-container/
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'get_archive',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Get a tar archive of a path of container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/get_archive',
                'method': 'GET'
            }
        ]
    ),
    # FIXME(lbragstad): This API call isn't actually listed in zun's API
    # reference, verify with someone from zun:
    # https://docs.openstack.org/api-ref/application-container/
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'put_archive',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Put a tar archive to be extracted to a path of container',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/put_archive',
                'method': 'PUT'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'stats',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Display the statistics of a container',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/stats',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'commit',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Commit a container',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/commit',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'add_security_group',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Add a security group to a specific container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/add_security_group',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'network_detach',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Detach a network from a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/network_detach',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'network_attach',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Attach a network from a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/network_attach',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'remove_security_group',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Remove security group from a specific container.',
        operations=[
            {
                'path': ('/v1/containers/'
                         '{container_ident}/remove_security_group'),
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'rebuild',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Rebuild a container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/rebuild',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=CONTAINER % 'resize_container',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Resize an existing  container.',
        operations=[
            {
                'path': '/v1/containers/{container_ident}/resize_container',
                'method': 'POST'
            }
        ]
    ),
]


def list_rules():
    return rules



# Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from oslo_config import cfg

import webtest

from zun.api import app
from zun.tests.unit.api import base as api_base


CURRENT_VERSION = api_base.CURRENT_VERSION


class TestRootController(api_base.FunctionalTest):
    def setUp(self):
        super(TestRootController, self).setUp()
        self.root_expected = {
            'default_version':
            {'id': 'v1',
             'links': [{'href': 'http://localhost/v1/', 'rel': 'self'}],
             'max_version': '1.40',
             'min_version': '1.1',
             'status': 'CURRENT'},
            'description': 'Zun is an OpenStack project which '
            'aims to provide containers service.',
            'name': 'OpenStack Zun API',
            'versions': [{'id': 'v1',
                          'links': [{'href': 'http://localhost/v1/',
                                     'rel': 'self'}],
                          'max_version': '1.40',
                          'min_version': '1.1',
                          'status': 'CURRENT'}]}

        self.v1_expected = {
            'media_types':
            [{'base': 'application/json',
              'type': 'application/vnd.openstack.zun.v1+json'}],
            'links': [{'href': 'http://localhost/v1/',
                       'rel': 'self'},
                      {'href':
                       'https://docs.openstack.org/developer'
                       '/zun/dev/api-spec-v1.html',
                       'type': 'text/html', 'rel': 'describedby'}],
            'services': [{'href': 'http://localhost/v1/services/',
                          'rel': 'self'},
                         {'href': 'http://localhost/services/',
                          'rel': 'bookmark'}],
            'id': 'v1',
            'containers': [{'href': 'http://localhost/v1/containers/',
                            'rel': 'self'},
                           {'href': 'http://localhost/containers/',
                            'rel': 'bookmark'}],
            'hosts': [{'href': 'http://localhost/v1/hosts/',
                       'rel': 'self'},
                      {'href': 'http://localhost/hosts/',
                       'rel': 'bookmark'}],
            'availability_zones': [
                {'href': 'http://localhost/v1/availability_zones/',
                 'rel': 'self'},
                {'href': 'http://localhost/availability_zones/',
                 'rel': 'bookmark'}],
            'images': [{'href': 'http://localhost/v1/images/',
                        'rel': 'self'},
                       {'href': 'http://localhost/images/',
                        'rel': 'bookmark'}],
            'networks': [{'href': 'http://localhost/v1/networks/',
                          'rel': 'self'},
                         {'href': 'http://localhost/networks/',
                          'rel': 'bookmark'}],
            'capsules': [{'href': 'http://localhost/v1/capsules/',
                          'rel': 'self'},
                         {'href': 'http://localhost/capsules/',
                          'rel': 'bookmark'}],
            'quotas': [{'href': 'http://localhost/v1/quotas/',
                        'rel': 'self'},
                       {'href': 'http://localhost/quotas/',
                        'rel': 'bookmark'}],
            'quota_classes': [{'href': 'http://localhost/v1/quota_classes/',
                               'rel': 'self'},
                              {'href': 'http://localhost/quota_classes/',
                               'rel': 'bookmark'}],
            'registries': [{'href': 'http://localhost/v1/registries/',
                            'rel': 'self'},
                           {'href': 'http://localhost/registries/',
                            'rel': 'bookmark'}],
        }

    def make_app(self, paste_file):
        file_name = self.get_path(paste_file)
        cfg.CONF.set_override("api_paste_config", file_name, group="api")
        return webtest.TestApp(app.load_app())

    def test_version(self):
        response = self.app.get('/')
        self.assertEqual(self.root_expected, response.json)

    def test_v1_controller(self):
        response = self.app.get('/v1/')
        self.assertEqual(self.v1_expected, response.json)

    def test_get_not_found(self):
        response = self.app.get('/a/bogus/url', expect_errors=True)
        assert response.status_int == 404

    def test_noauth(self):
        # Don't need to auth
        paste_file = "zun/tests/unit/api/controllers/noauth-paste.ini"
        headers = {'OpenStack-API-Version': CURRENT_VERSION}
        app = self.make_app(paste_file)

        response = app.get('/', headers=headers)
        self.assertEqual(self.root_expected, response.json)

        response = app.get('/v1/', headers=headers)
        self.assertEqual(self.v1_expected, response.json)

        response = app.get('/v1/containers/', headers=headers)
        self.assertEqual(200, response.status_int)

    def test_auth_with_no_public_routes(self):
        # All apis need auth when access
        paste_file = "zun/tests/unit/api/controllers/auth-paste.ini"
        app = self.make_app(paste_file)

        response = app.get('/', expect_errors=True)
        self.assertEqual(401, response.status_int)

        response = app.get('/v1/', expect_errors=True)
        self.assertEqual(401, response.status_int)

    def test_auth_with_root_access(self):
        # Only / can access without auth
        paste_file = "zun/tests/unit/api/controllers/auth-root-access.ini"
        app = self.make_app(paste_file)

        response = app.get('/')
        self.assertEqual(self.root_expected, response.json)

        response = app.get('/v1/', expect_errors=True)
        self.assertEqual(401, response.status_int)

        response = app.get('/v1/containers', expect_errors=True)
        self.assertEqual(401, response.status_int)

    def test_auth_with_v1_access(self):
        # Only /v1 can access without auth
        paste_file = "zun/tests/unit/api/controllers/auth-v1-access.ini"
        app = self.make_app(paste_file)

        response = app.get('/', expect_errors=True)
        self.assertEqual(401, response.status_int)

        response = app.get('/v1/')
        self.assertEqual(self.v1_expected, response.json)

        response = app.get('/v1/containers', expect_errors=True)
        self.assertEqual(401, response.status_int)



# Copyright 2017 IBM Corp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from collections import defaultdict
import errno
import re

from oslo_log import log as logging
from zun.common import exception
from zun.common import utils
from zun.container.os_capability import host_capability


LOG = logging.getLogger(__name__)


class LinuxHost(host_capability.Host):

    def get_cpu_numa_info(self):
        # TODO(sbiswas7): rootwrap changes for zun required.
        old_lscpu = False
        try:
            output = utils.execute('lscpu', '-p=socket,cpu,online')
        except exception.CommandError:
            LOG.info("There was a problem while executing lscpu -p=socket"
                     ",cpu,online. Try again without the online column.")
            # There is a possibility that an older version of lscpu is used
            # So let's try without the online column
            output = utils.execute('lscpu', '-p=socket,cpu')
            old_lscpu = True

        if old_lscpu:
            cpu_sock_pair = re.findall(r"\d+(?:,\d+)?", str(output))
        else:
            cpu_sock_pair = re.findall(r"\d+(?:,\d+,[Y/N])?", str(output))
        sock_map = defaultdict(list)
        for value in cpu_sock_pair:
            val = value.split(",")
            if len(val) == 3 and val[2] == 'Y':
                sock_map[val[0]].append(int(val[1]))
            elif len(val) == 2 and old_lscpu:
                sock_map[val[0]].append(int(val[1]))
        return sock_map

    def get_mem_numa_info(self):
        try:
            output = utils.execute('numactl', '-H')
        except OSError as e:
            if e.errno == errno.ENOENT:
                LOG.info("The program 'numactl' is not installed.")
                return []
            else:
                raise

        sizes = re.findall(r"size\: \d*", str(output))
        mem_numa = []
        for size in sizes:
            mem_numa.append(int(size.split(' ')[1]))
        return mem_numa



# Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from unittest.mock import patch

from zun.tests.unit.api import base as api_base
from zun.tests.unit.db import utils


class TestNetworkController(api_base.FunctionalTest):
    @patch('zun.common.policy.enforce')
    @patch('zun.compute.api.API.network_create')
    def test_network_create(self, mock_network_create, mock_policy):
        mock_policy.return_value = True
        test_object = utils.create_test_network(context=self.context)
        mock_network_create.return_value = test_object
        params = ('{"name": "network-test", "neutron_net_id": "test-id"}')
        response = self.post('/v1/networks/',
                             params=params,
                             content_type='application/json')

        self.assertEqual(200, response.status_int)
        self.assertTrue(mock_network_create.called)

    @patch('zun.objects.ZunNetwork.list')
    @patch('zun.common.policy.enforce')
    @patch('zun.compute.api.API.network_delete')
    def test_network_delete(self, mock_network_delete, mock_policy,
                            mock_network_list):
        mock_policy.return_value = True
        test_object = utils.create_test_network(context=self.context)
        mock_network_list.return_value = [test_object]
        response = self.delete('/v1/networks/%s' % test_object.get('uuid'))
        self.assertEqual(204, response.status_int)
        self.assertTrue(mock_network_delete.called)



# Copyright 2016 Intel.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import shutil
import tempfile
from unittest import mock

from zun.common import exception
import zun.conf
from zun.image.glance import driver
from zun.tests import base

CONF = zun.conf.CONF


class TestDriver(base.BaseTestCase):
    def setUp(self):
        super(TestDriver, self).setUp()
        self.driver = driver.GlanceDriver()
        self.test_dir = tempfile.mkdtemp()

    def tearDown(self):
        # Remove the directory after the test
        super(TestDriver, self).tearDown()
        shutil.rmtree(self.test_dir)

    @mock.patch.object(driver.GlanceDriver,
                       '_search_image_on_host')
    @mock.patch('zun.common.utils.should_pull_image')
    def test_pull_image_should_pull_no_image_not_present_locally(
            self, mock_should_pull_image, mock_search):
        mock_should_pull_image.return_value = False
        mock_search.return_value = None
        self.assertRaises(exception.ImageNotFound, self.driver.pull_image,
                          None, 'nonexisting', 'tag', 'never', None)

    @mock.patch.object(driver.GlanceDriver,
                       '_search_image_on_host')
    @mock.patch('zun.common.utils.should_pull_image')
    def test_pull_image_should_pull_no_image_present_locally(
            self, mock_should_pull_image, mock_search):
        mock_should_pull_image.return_value = False
        checksum = 'd41d8cd98f00b204e9800998ecf8427e'
        mock_search.return_value = {'image': 'nginx', 'path': 'xyz',
                                    'checksum': checksum}
        mock_open_file = mock.mock_open()
        with mock.patch('zun.image.glance.driver.open', mock_open_file):
            self.assertEqual(({'image': 'nginx', 'path': 'xyz',
                               'checksum': checksum}, True),
                             self.driver.pull_image(None, 'nonexisting',
                                                    'tag', 'never', None))
        mock_open_file.assert_any_call('xyz', 'rb')

    @mock.patch.object(driver.GlanceDriver,
                       '_search_image_on_host')
    @mock.patch('zun.common.utils.should_pull_image')
    @mock.patch('zun.image.glance.utils.find_image')
    def test_pull_image_failure(self, mock_find_image,
                                mock_should_pull_image,
                                mock_search):
        mock_should_pull_image.return_value = True
        mock_search.return_value = {'image': 'nginx', 'path': 'xyz',
                                    'checksum': 'xxx'}
        mock_find_image.side_effect = Exception
        self.assertRaises(exception.ZunException, self.driver.pull_image,
                          None, 'nonexisting', 'tag', 'always', None)

    @mock.patch.object(driver.GlanceDriver,
                       '_search_image_on_host')
    @mock.patch('zun.common.utils.should_pull_image')
    @mock.patch('zun.image.glance.utils.download_image_in_chunks')
    @mock.patch('zun.image.glance.utils.find_image')
    def test_pull_image_success(self, mock_find_image, mock_download_image,
                                mock_should_pull_image, mock_search_on_host):
        mock_should_pull_image.return_value = True
        mock_search_on_host.return_value = {'image': 'nginx', 'path': 'xyz',
                                            'checksum': 'xxx'}
        image_meta = mock.MagicMock()
        image_meta.id = '1234'
        image_meta.name = 'image'
        image_meta.tags = ['latest']
        mock_find_image.return_value = image_meta
        mock_download_image.return_value = 'content'
        CONF.set_override('images_directory', self.test_dir, group='glance')
        out_path = os.path.join(self.test_dir, '1234' + '.tar')
        mock_open_file = mock.mock_open()
        with mock.patch('zun.image.glance.driver.open', mock_open_file):
            ret = self.driver.pull_image(None, 'image', 'latest', 'always',
                                         None)
        mock_open_file.assert_any_call(out_path, 'wb')
        self.assertTrue(mock_search_on_host.called)
        self.assertTrue(mock_should_pull_image.called)
        self.assertTrue(mock_find_image.called)
        self.assertTrue(mock_download_image.called)
        self.assertEqual(({'image': 'image', 'path': out_path,
                           'tags': ['latest']}, False), ret)

    @mock.patch('zun.common.utils.should_pull_image')
    def test_pull_image_not_found(self, mock_should_pull_image):
        mock_should_pull_image.return_value = True
        with mock.patch('zun.image.glance.utils.find_image') as mock_find:
            mock_find.side_effect = exception.ImageNotFound
            self.assertRaises(exception.ImageNotFound, self.driver.pull_image,
                              None, 'nonexisting', 'tag', 'always', None)

    @mock.patch('zun.image.glance.utils.find_images')
    def test_search_image_found(self, mock_find_images):
        image_meta = mock.MagicMock()
        image_meta.id = '1234'
        mock_find_images.return_value = [image_meta]
        ret = self.driver.search_image(None, 'image', None, False)
        self.assertEqual(1, len(ret))
        self.assertTrue(mock_find_images.called)

    @mock.patch('zun.image.glance.utils.find_images')
    def test_search_image_not_found(self, mock_find_images):
        image_meta = mock.MagicMock()
        image_meta.id = '1234'
        mock_find_images.return_value = []
        ret = self.driver.search_image(None, 'image', None, False)
        self.assertEqual(0, len(ret))
        self.assertTrue(mock_find_images.called)

    @mock.patch('zun.image.glance.utils.find_images')
    def test_search_image_exception(self, mock_find_images):
        mock_find_images.side_effect = Exception
        self.assertRaises(exception.ZunException, self.driver.search_image,
                          None, 'image', None, False)
        self.assertTrue(mock_find_images.called)

    @mock.patch('zun.image.glance.utils.create_image')
    def test_create_image(self, mock_create_image):
        image_meta = mock.MagicMock()
        image_meta.id = '1234'
        mock_create_image.return_value = [image_meta]
        ret = self.driver.create_image(None, 'image')
        self.assertEqual(1, len(ret))
        self.assertTrue(mock_create_image.called)

    @mock.patch('zun.image.glance.utils.update_image')
    def test_update_image(self, mock_update_image):
        image_meta = mock.MagicMock()
        image_meta.id = '1234'
        mock_update_image.return_value = [image_meta]
        ret = self.driver.update_image(None, 'id', container_format='docker')
        self.assertEqual(1, len(ret))
        self.assertTrue(mock_update_image.called)

    @mock.patch('zun.image.glance.utils.delete_image')
    def test_delete_committed_image(self, mock_delete_image):
        image_meta = mock.MagicMock()
        image_meta.id = '1234'
        mock_delete_image.return_value = [image_meta]
        ret = self.driver.delete_committed_image(None, 'id')
        self.assertEqual(1, len(ret))
        self.assertTrue(mock_delete_image.called)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""add healthcheck to containers

Revision ID: 2fb377a5a519
Revises: 105626c4f972
Create Date: 2018-05-03 11:27:00.722445

"""

# revision identifiers, used by Alembic.
revision = '2fb377a5a519'
down_revision = '105626c4f972'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import zun


def upgrade():
    with op.batch_alter_table('container', schema=None) as batch_op:
        batch_op.add_column(sa.Column(
            'healthcheck', zun.db.sqlalchemy.models.JSONEncodedDict(),
            nullable=True))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from os_vif.objects import base
from oslo_config import cfg
from oslo_log import log as logging

from zun.common import consts


CONF = cfg.CONF
LOG = logging.getLogger(__name__)


def get_vifs(container):
    try:
        cni_metadata = container.cni_metadata
        vif_state = cni_metadata[consts.CNI_METADATA_VIF]
    except KeyError:
        return {}
    vif_state = base.VersionedObject.obj_from_primitive(vif_state)
    vifs_dict = vif_state.vifs
    LOG.debug("Got VIFs from cni_metadata: %r", vifs_dict)
    return vifs_dict


def convert_netns(netns):
    """Convert /proc based netns path to Docker-friendly path.

    When CONF.docker_mode is set this method will change /proc to
    /CONF.cni_daemon.cni_daemon.netns_proc_dir. This allows netns manipulations
    to work when running in Docker container on Zun host.

    :param netns: netns path to convert.
    :return: Converted netns path.
    """
    if CONF.cni_daemon.docker_mode:
        return netns.replace('/proc', CONF.cni_daemon.netns_proc_dir)
    else:
        return netns


def _osvif_route_to_dict(route):
    return {
        'gateway': str(route.gateway),
        'cidr': str(route.cidr),
    }


def _osvif_fip_to_dict(fix):
    return {
        'address': str(fix.address),
    }


def _osvif_subnet_to_dict(subnet):
    subnet_dict = {
        'routes': [_osvif_route_to_dict(r)
                   for r in subnet.routes.objects],
        'ips': [_osvif_fip_to_dict(ip) for ip in subnet.ips],
        'cidr': {
            'version': subnet.cidr.version,
            'prefixlen': subnet.cidr.prefixlen,
        },
    }
    if subnet.obj_attr_is_set('gateway'):
        subnet_dict['gateway'] = str(subnet.gateway)
    return subnet_dict


def _osvif_network_to_dict(network):
    return {
        'subnets': [_osvif_subnet_to_dict(s)
                    for s in network.subnets.objects],
        'mtu': network.mtu,
    }


def osvif_vif_to_dict(vif):
    return {
        'network': _osvif_network_to_dict(vif.network),
        'vif_name': vif.vif_name,
        'address': str(vif.address),
        'bridge_name': vif.bridge_name,
    }


class CNIConfig(dict):
    def __init__(self, cfg):
        super(CNIConfig, self).__init__(cfg)

        for k, v in self.items():
            if not k.startswith('_'):
                setattr(self, k, v)


class CNIArgs(object):
    def __init__(self, value):
        for item in value.split(';'):
            k, v = item.split('=', 1)
            if not k.startswith('_'):
                setattr(self, k, v)


class CNIParameters(object):
    def __init__(self, env, cfg=None):
        for k, v in env.items():
            if k.startswith('CNI_'):
                setattr(self, k, v)
        if cfg is None:
            self.config = CNIConfig(env['config_zun'])
        else:
            self.config = cfg
        self.args = CNIArgs(self.CNI_ARGS)

    def __repr__(self):
        return repr({key: value for key, value in self.__dict__.items() if
                     key.startswith('CNI_')})



# -*- encoding: utf-8 -*-
#
# Copyright © 2012 eNovance <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from oslo_config import cfg


path_opts = [
    cfg.StrOpt('pybasedir',
               default=os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                    '../')),
               help='Directory where the zun python module is installed.'),
    cfg.StrOpt('bindir',
               default='$pybasedir/bin',
               help='Directory where zun binaries are installed.'),
    cfg.StrOpt('state_path',
               default='$pybasedir',
               help="Top-level directory for maintaining zun's state."),
]


def state_path_def(*args):
    """Return an uninterpolated path relative to $state_path."""
    return os.path.join('$state_path', *args)


def register_opts(conf):
    conf.register_opts(path_opts)


def list_opts():
    return {"DEFAULT": path_opts}



# Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""add annotations to container

Revision ID: df87dbd4846c
Revises: d502ce8fb705
Create Date: 2019-12-21 16:47:11.503444

"""

# revision identifiers, used by Alembic.
revision = 'df87dbd4846c'
down_revision = 'd502ce8fb705'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def MediumText():
    return sa.Text().with_variant(sa.dialects.mysql.MEDIUMTEXT(), 'mysql')


def upgrade():
    op.add_column('container',
                  sa.Column('annotations', MediumText(), nullable=True))



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from testtools.matchers import HasLength

from zun import objects
from zun.tests.unit.db import base
from zun.tests.unit.db import utils


class TestImageObject(base.DbTestCase):

    def setUp(self):
        super(TestImageObject, self).setUp()
        self.fake_image = utils.get_test_image()

    def test_get_by_uuid(self):
        uuid = self.fake_image['uuid']
        with mock.patch.object(self.dbapi, 'get_image_by_uuid',
                               autospec=True) as mock_get_image:
            mock_get_image.return_value = self.fake_image
            image = objects.Image.get_by_uuid(self.context, uuid)
            mock_get_image.assert_called_once_with(self.context, uuid)
            self.assertEqual(self.context, image._context)

    def test_list(self):
        with mock.patch.object(self.dbapi, 'list_images',
                               autospec=True) as mock_get_list:
            mock_get_list.return_value = [self.fake_image]
            images = objects.Image.list(self.context)
            self.assertEqual(1, mock_get_list.call_count)
            self.assertThat(images, HasLength(1))
            self.assertIsInstance(images[0], objects.Image)
            self.assertEqual(self.context, images[0]._context)

    def test_list_with_filters(self):
        with mock.patch.object(self.dbapi, 'list_images',
                               autospec=True) as mock_get_list:
            mock_get_list.return_value = [self.fake_image]
            filt = {'id': '1'}
            images = objects.Image.list(self.context, filters=filt)
            self.assertEqual(1, mock_get_list.call_count)
            self.assertThat(images, HasLength(1))
            self.assertIsInstance(images[0], objects.Image)
            self.assertEqual(self.context, images[0]._context)
            mock_get_list.assert_called_once_with(self.context,
                                                  filters=filt,
                                                  limit=None, marker=None,
                                                  sort_key=None, sort_dir=None)

    def test_pull(self):
        with mock.patch.object(self.dbapi, 'pull_image',
                               autospec=True) as mock_pull_image:
            mock_pull_image.return_value = self.fake_image
            image = objects.Image(self.context, **self.fake_image)
            image.pull(self.context)
            mock_pull_image.assert_called_once_with(self.context,
                                                    self.fake_image)
            self.assertEqual(self.context, image._context)

    def test_save(self):
        uuid = self.fake_image['uuid']
        with mock.patch.object(self.dbapi, 'get_image_by_uuid',
                               autospec=True) as mock_get_image:
            mock_get_image.return_value = self.fake_image
            with mock.patch.object(self.dbapi, 'update_image',
                                   autospec=True) as mock_update_image:
                image = objects.Image.get_by_uuid(self.context, uuid)
                image.repo = 'image-test'
                image.tag = '512'
                image.save()

                mock_get_image.assert_called_once_with(self.context, uuid)
                mock_update_image.assert_called_once_with(uuid,
                                                          {'repo':
                                                           'image-test',
                                                           'tag': '512'})
                self.assertEqual(self.context, image._context)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils
from oslo_versionedobjects import fields

from zun.db import api as dbapi
from zun.objects import base
from zun.objects.numa import NUMATopology
from zun.objects import pci_device_pool


@base.ZunObjectRegistry.register
class ComputeNode(base.ZunPersistentObject, base.ZunObject):
    # Version 1.0: Initial version
    # Version 1.1: Add mem_total, mem_free, mem_available columns
    # Version 1.2: Add total, running, pasued, stopped containers columns
    # Version 1.3: Add cpus, cpu_used
    # Version 1.4: Add host operating system info
    # Version 1.5: Add host labels info
    # Version 1.6: Add mem_used to compute node
    # Version 1.7: Change get_by_hostname to get_by_name
    # Version 1.8: Add pci_device_pools to compute node
    # Version 1.9: Change PciDevicePoolList to ObjectField
    # Version 1.10: Add disk_total, disk_used columns
    # Version 1.11: Add disk_quota_supported field
    # Version 1.12: Add runtimes field
    # Version 1.13: Add enable_cpu_pinning field
    # Version 1.14: Add rp_uuid field
    VERSION = '1.14'

    fields = {
        'uuid': fields.UUIDField(read_only=True, nullable=False),
        'rp_uuid': fields.UUIDField(nullable=False),
        'numa_topology': fields.ObjectField('NUMATopology', nullable=True),
        'hostname': fields.StringField(nullable=False),
        'mem_total': fields.IntegerField(nullable=False),
        'mem_free': fields.IntegerField(nullable=False),
        'mem_available': fields.IntegerField(nullable=False),
        'mem_used': fields.IntegerField(nullable=False),
        'total_containers': fields.IntegerField(nullable=False),
        'running_containers': fields.IntegerField(nullable=False),
        'paused_containers': fields.IntegerField(nullable=False),
        'stopped_containers': fields.IntegerField(nullable=False),
        'cpus': fields.IntegerField(nullable=False),
        'cpu_used': fields.FloatField(nullable=False),
        'architecture': fields.StringField(nullable=True),
        'os_type': fields.StringField(nullable=True),
        'os': fields.StringField(nullable=True),
        'kernel_version': fields.StringField(nullable=True),
        'labels': fields.DictOfStringsField(nullable=True),
        # NOTE(pmurray): the pci_device_pools field maps to the
        # pci_stats field in the database
        'pci_device_pools': fields.ObjectField('PciDevicePoolList',
                                               nullable=True),
        'disk_total': fields.IntegerField(nullable=False),
        'disk_used': fields.IntegerField(nullable=False),
        'disk_quota_supported': fields.BooleanField(nullable=False),
        'runtimes': fields.ListOfStringsField(nullable=True),
        'enable_cpu_pinning': fields.BooleanField(nullable=False),
    }

    @staticmethod
    def _from_db_object(context, compute_node, db_compute_node):
        """Converts a database entity to a formal object."""
        special_cases = set(['pci_device_pools'])
        fields = set(compute_node.fields) - special_cases
        for field in fields:
            if field == 'numa_topology':
                numa_obj = NUMATopology._from_dict(
                    db_compute_node['numa_topology'])
                compute_node.numa_topology = numa_obj
            else:
                setattr(compute_node, field, db_compute_node[field])

        pci_stats = db_compute_node.get('pci_stats')
        if pci_stats is not None:
            pci_stats = pci_device_pool.from_pci_stats(pci_stats)
        compute_node.pci_device_pools = pci_stats
        compute_node.obj_reset_changes(recursive=True)
        return compute_node

    @staticmethod
    def _from_db_object_list(db_objects, cls, context):
        """Converts a list of database entities to a list of formal objects."""
        return [ComputeNode._from_db_object(context, cls(context), obj)
                for obj in db_objects]

    @staticmethod
    def _convert_pci_stats_to_db_format(updates):
        if 'pci_device_pools' in updates:
            pools = updates.pop('pci_device_pools')
            if pools is not None:
                pools = jsonutils.dumps(pools.obj_to_primitive())
            updates['pci_stats'] = pools

    @base.remotable
    def create(self, context):
        """Create a compute node record in the DB.

        :param context: Security context.

        """
        values = self.obj_get_changes()
        numa_obj = values.pop('numa_topology', None)
        if numa_obj is not None:
            values['numa_topology'] = numa_obj._to_dict()

        self._convert_pci_stats_to_db_format(values)
        db_compute_node = dbapi.create_compute_node(context, values)
        self._from_db_object(context, self, db_compute_node)

    @base.remotable_classmethod
    def get_by_uuid(cls, context, uuid):
        """Find a compute node based on uuid.

        :param uuid: the uuid of a compute node.
        :param context: Security context
        :returns: a :class:`ComputeNode` object.
        """
        db_compute_node = dbapi.get_compute_node(context, uuid)
        compute_node = ComputeNode._from_db_object(
            context, cls(context), db_compute_node)
        return compute_node

    @base.remotable_classmethod
    def get_by_name(cls, context, hostname):
        db_compute_node = dbapi.get_compute_node_by_hostname(
            context, hostname)
        return cls._from_db_object(context, cls(), db_compute_node)

    @base.remotable_classmethod
    def list(cls, context, limit=None, marker=None,
             sort_key=None, sort_dir=None, filters=None):
        """Return a list of ComputeNode objects.

        :param context: Security context.
        :param limit: maximum number of resources to return in a single result.
        :param marker: pagination marker for large data sets.
        :param sort_key: column to sort results by.
        :param sort_dir: direction to sort. "asc" or "desc".
        :param filters: filters when list resource providers.
        :returns: a list of :class:`ComputeNode` object.

        """
        db_compute_nodes = dbapi.list_compute_nodes(
            context, limit=limit, marker=marker, sort_key=sort_key,
            sort_dir=sort_dir, filters=filters)
        return ComputeNode._from_db_object_list(
            db_compute_nodes, cls, context)

    @base.remotable
    def destroy(self, context=None):
        """Delete the ComputeNode from the DB.

        :param context: Security context.
        """
        dbapi.destroy_compute_node(context, self.uuid)
        self.obj_reset_changes(recursive=True)

    @base.remotable
    def save(self, context=None):
        """Save updates to this ComputeNode.

        Updates will be made column by column based on the result
        of self.what_changed().

        :param context: Security context.
        """
        updates = self.obj_get_changes()
        numa_obj = updates.pop('numa_topology', None)
        if numa_obj is not None:
            updates['numa_topology'] = numa_obj._to_dict()
        self._convert_pci_stats_to_db_format(updates)
        dbapi.update_compute_node(context, self.uuid, updates)
        self.obj_reset_changes(recursive=True)

    @base.remotable
    def refresh(self, context=None):
        """Loads updates for this ComputeNode.

        Loads a compute node with the same uuid from the database and
        checks for updated attributes. Updates are applied from
        the loaded compute node column by column, if there are any
        updates.

        :param context: Security context.
        """
        current = self.__class__.get_by_uuid(self._context, uuid=self.uuid)
        for field in self.fields:
            if self.obj_attr_is_set(field) and \
               getattr(self, field) != getattr(current, field):
                setattr(self, field, getattr(current, field))



#    Copyright 2016 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from zun.compute import claims
from zun.compute import compute_node_tracker
from zun import objects
from zun.tests import base
from zun.tests.unit.container import fake_driver
from zun.tests.unit.objects import utils as obj_utils


class TestNodeStracker(base.TestCase):

    def setUp(self):
        super(TestNodeStracker, self).setUp()
        self.container_driver = fake_driver.FakeDriver()
        self.capsule_driver = fake_driver.FakeDriver()
        self.report_client_mock = mock.MagicMock()
        self._resource_tracker = compute_node_tracker.ComputeNodeTracker(
            'testhost', self.container_driver, self.capsule_driver,
            self.report_client_mock)

    @mock.patch.object(compute_node_tracker.ComputeNodeTracker, '_update')
    @mock.patch.object(compute_node_tracker.ComputeNodeTracker,
                       '_update_usage_from_container_update')
    @mock.patch.object(claims, 'UpdateClaim')
    def test_container_update_same_resource(self, mock_claim,
                                            mock_container_update,
                                            mock_update):
        container1 = obj_utils.get_test_container(
            self.context, cpu=1, memory=1024)
        container2 = obj_utils.get_test_container(
            self.context, cpu=1, memory=1024)

        self._resource_tracker.container_update_claim(
            self.context, container1, container2)
        self.assertFalse(mock_claim.called)
        self.assertFalse(mock_container_update.called)
        self.assertFalse(mock_update.called)

    @mock.patch.object(
        compute_node_tracker.ComputeNodeTracker, '_get_compute_node')
    @mock.patch.object(compute_node_tracker.ComputeNodeTracker, '_update')
    @mock.patch.object(compute_node_tracker.ComputeNodeTracker,
                       '_update_usage_from_container_update')
    @mock.patch.object(claims, 'UpdateClaim')
    def test_container_update(self, mock_claim, mock_container_update,
                              mock_update, mock_get_node):
        container1 = obj_utils.get_test_container(
            self.context, cpu=1, memory=1024)
        container2 = obj_utils.get_test_container(
            self.context, cpu2=2, memory=2048)
        node = objects.ComputeNode(self.context)
        node.cpu = 10
        node.memory = 3072
        mock_get_node.return_value = node

        self._resource_tracker.container_update_claim(
            self.context, container1, container2)
        self.assertTrue(mock_claim.called)
        self.assertTrue(mock_container_update.called)
        self.assertTrue(mock_update.called)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_versionedobjects import fields

from zun.db import api as dbapi
from zun.objects import base


@base.ZunObjectRegistry.register
class Quota(base.ZunPersistentObject, base.ZunObject):
    # Version 1.0: Initial version
    # Version 1.1: Add uuid column
    # Version 1.2: Add destroy_all_by_project method
    # Version 1.3: Remove uuid column
    VERSION = '1.3'

    fields = {
        'id': fields.IntegerField(),
        'project_id': fields.StringField(nullable=True),
        'resource': fields.StringField(),
        'hard_limit': fields.IntegerField(nullable=True),
    }

    @staticmethod
    def _from_db_object(quota, db_quota):
        """Converts a database entity to a formal object"""
        for field in quota.fields:
            if field == 'uuid':
                continue
            setattr(quota, field, db_quota[field])

        quota.obj_reset_changes()
        return quota

    @base.remotable_classmethod
    def get(cls, context, project_id, resource):
        """Find a quota based on project_id and resource

        :param project_id: the project id.
        :param context: security context.
        :param resource: the name of resource.
        :returns: a :class:`Quota` object.
        """
        db_quota = dbapi.quota_get(context, project_id, resource)
        quota = Quota._from_db_object(cls(context), db_quota)
        return quota

    @base.remotable_classmethod
    def get_all(cls, context, project_id):
        """Find all quotas associated with project

        :param context: security context.
        :param project_id: the project id.
        :returns: a dict
        """
        return dbapi.quota_get_all_by_project(context, project_id)

    @base.remotable
    def create(self, context):
        """Create a Quota record in the DB.

        :param context: security context. NOTE: This should only be
                        used internally by the indirection api.
                        Unfortunately, RPC requires context as the first
                        argument, even though we don't use it.
                        A context should be set when instantiating the
                        object, e.g.: Quota(context)
        """
        values = self.obj_get_changes()
        project_id = values.get('project_id')
        resource = values.get('resource')
        limit = values.get('hard_limit')
        db_quota = dbapi.quota_create(context, project_id, resource, limit)
        self._from_db_object(self, db_quota)

    @base.remotable_classmethod
    def destroy_all_by_project(cls, context, project_id):
        """Destroy all quotas associated with a project.

        :param context: security context.
        :param project_id: the id of the project
        """
        dbapi.quota_destroy_all_by_project(context, project_id)

    @base.remotable
    def destroy(self, context=None):
        """Delete the Quota from the DB.

        :param context: security context. NOTE: This should only be
                        used internally by the indirection api.
                        Unfortunately, RPC requires context as the first
                        argument, even though we don't use it.
                        A context should be set when instantiating the
                        object, e.g.: Quota(context)
        """
        dbapi.quota_destroy(context, self.project_id, self.resource)
        self.obj_reset_changes()

    @base.remotable
    def update(self, context=None):
        """Save updates to this Quota.

        Updates will be made column by column based on the result
        of self.what_changed().

        :param context: security context. NOTE: This should only be
                        used internally by the indirection api.
                        Unfortunately, RPC requires context as the first
                        argument, even though we don't use it.
                        A context should be set when instantiating the
                        object, e.g.: Quota(context)
        """
        updates = self.obj_get_changes()
        dbapi.quota_update(context, self.project_id, self.resource,
                           updates.get('hard_limit'))
        self.obj_reset_changes()



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import itertools

from zun.common import utils


_action_keys = (
    'action',
    'container_uuid',
    'finish_time',
    'message',
    'project_id',
    'request_id',
    'start_time',
    'user_id',
)


_action_event_keys = (
    'event',
    'finish_time',
    'result',
    'start_time',
    'traceback',
)


def format_action(action):
    def transform(key, value):
        if key not in _action_keys:
            return

        if isinstance(value, datetime.datetime):
            yield (key, utils.strtime(value))
        else:
            yield (key, value)

    return dict(itertools.chain.from_iterable(
        transform(k, v) for k, v in action.as_dict().items()))


def format_event(event, show_traceback=False):
    def transform(key, value):
        if key not in _action_event_keys:
            return

        if isinstance(value, datetime.datetime):
            yield (key, utils.strtime(value))
        else:
            if key == 'traceback' and not show_traceback:
                # By default, non-admins are not allowed to see traceback
                # details.
                yield (key, None)
            else:
                yield (key, value)

    return dict(itertools.chain.from_iterable(
        transform(k, v) for k, v in event.as_dict().items()))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools

from oslo_log.log import logging
from oslo_utils import timeutils

from zun.common import utils
from zun.pci import stats as pci_stats

LOG = logging.getLogger(__name__)


class HostState(object):
    """Mutable and immutable information tracked for a host.

    This is an attempt to remove the ad-hoc data structures.
    """

    def __init__(self, host):
        self.hostname = host
        self._lock_name = host
        self.uuid = None

        # Mutable available resources.
        # These will change as resources are virtually "consumed".
        self.mem_available = 0
        self.mem_total = 0
        self.mem_free = 0
        self.mem_used = 0
        self.cpus = 0
        self.cpu_used = 0
        self.disk_total = 0
        self.disk_used = 0
        self.numa_topology = None
        self.labels = None
        self.pci_stats = None
        self.disk_quota_supported = False
        self.runtimes = []
        self.enable_cpu_pinning = False

        # Resource oversubscription values for the compute host:
        self.limits = {}

        self.updated = None

    def update(self, compute_node=None, service=None):
        """Update information about a host"""
        @utils.synchronized((self.hostname, compute_node))
        def _locked_update(self, compute_node, service):
            if compute_node is not None:
                LOG.debug('Update host state from compute node: %s',
                          compute_node)
                self._update_from_compute_node(compute_node)
            if service is not None:
                LOG.debug('Update host state with service: %s', service)
                self.service = service

        return _locked_update(self, compute_node, service)

    def _update_from_compute_node(self, compute_node):
        """Update information about a host from a Compute object"""
        if (self.updated and compute_node.updated_at and
                self.updated > compute_node.updated_at):
            return

        self.uuid = compute_node.rp_uuid
        self.mem_available = compute_node.mem_available
        self.mem_total = compute_node.mem_total
        self.mem_free = compute_node.mem_free
        self.mem_used = compute_node.mem_used
        self.cpus = compute_node.cpus
        self.cpu_used = compute_node.cpu_used
        self.disk_total = compute_node.disk_total
        self.disk_used = compute_node.disk_used
        self.numa_topology = compute_node.numa_topology
        self.labels = compute_node.labels
        self.pci_stats = pci_stats.PciDeviceStats(
            stats=compute_node.pci_device_pools)
        self.disk_quota_supported = compute_node.disk_quota_supported
        self.runtimes = compute_node.runtimes
        self.enable_cpu_pinning = compute_node.enable_cpu_pinning
        self.updated = compute_node.updated_at

    def consume_from_request(self, container):
        """Incrementally update host state from a Container object."""

        @utils.synchronized(self._lock_name)
        @set_update_time_on_success
        def _locked(self, container):
            # Scheduler API is inherently multi-threaded as every incoming RPC
            # message will be dispatched in its own green thread. So the
            # shared host state should be consumed in a consistent way to make
            # sure its data is valid under concurrent write operations.
            self._locked_consume_from_request(container)

        return _locked(self, container)

    def _locked_consume_from_request(self, container):
        disk = container.disk if container.disk else 0
        ram_mb = int(container.memory) if container.memory else 0
        vcpus = container.cpu if container.cpu else 0
        self.mem_used += ram_mb
        self.disk_used += disk
        self.cpu_used += vcpus
        self.mem_free = self.mem_total - self.mem_used
        # TODO(hongbin): track numa_topology and pci devices

    def __repr__(self):
        return ("%(host)s ram: %(free_ram)sMB "
                "disk: %(free_disk)sGB cpus: %(free_cpu)s" %
                {'host': self.hostname,
                 'free_ram': self.mem_free,
                 'free_disk': self.disk_total - self.disk_used,
                 'free_cpu': self.cpus - self.cpu_used})


@utils.expects_func_args('self', 'container')
def set_update_time_on_success(function):
    """Set updated time of HostState when consuming succeed."""

    @functools.wraps(function)
    def decorated_function(self, container):
        return_value = None
        try:
            return_value = function(self, container)
        except Exception as e:
            # Ignores exception raised from consume_from_request() so that
            # booting container would fail in the resource claim of compute
            # node, other suitable node may be chosen during scheduling retry.
            LOG.warning("Selected host: %(host)s failed to consume from "
                        "container. Error: %(error)s",
                        {'host': self.hostname, 'error': e})
        else:
            self.updated = timeutils.utcnow()
        return return_value

    return decorated_function



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

from zun.common.i18n import _


cni_daemon_group = cfg.OptGroup(name='cni_daemon',
                                title='Options for zun-cni-daemon')

daemon_opts = [
    cfg.PortOpt('cni_daemon_port',
                default=9036,
                help=_('The port for the CNI daemon.')),
    cfg.IPOpt('cni_daemon_host',
              default='127.0.0.1',
              help=_('Bind address for CNI daemon HTTP server. It is '
                     'recommened to allow only local connections.')),
    cfg.IntOpt('worker_num',
               help=_('Maximum number of processes that will be spawned to '
                      'process requests from CNI driver.'),
               default=30),
    cfg.IntOpt('vif_active_timeout',
               help=_('Time (in seconds) the CNI daemon will wait for VIF '
                      'to be active.'),
               default=60),
    cfg.IntOpt('pyroute2_timeout',
               help=_('Zun uses pyroute2 library to manipulate networking '
                      'interfaces. When processing a high number of Zun '
                      'requests in parallel, it may take kernel more time to '
                      'process all networking stack changes. This option '
                      'allows to tune internal pyroute2 timeout.'),
               default=10),
    cfg.BoolOpt('docker_mode',
                help=_('Set to True when you are running zun-cni-daemon '
                       'inside a Docker container. This mainly means that '
                       'zun-cni-daemon will look for network namespaces in '
                       '$netns_proc_dir instead of /proc.'),
                default=False),
    cfg.StrOpt('netns_proc_dir',
               help=_("When docker_mode is set to True, this config option "
                      "should be set to where host's /proc directory is "
                      "mounted. Please note that mounting it is necessary to "
                      "allow Zun to move host interfaces between "
                      "host network namespaces, which is essential for Zun "
                      "to work."),
               default=None),
    cfg.DictOpt('sriov_physnet_resource_mappings',
                help=_("A mapping of physnets for certain sriov dp "
                       "resource name in a form of "
                       "physnet-name:resource name. "
                       "Resource name is listed in sriov device plugin "
                       "configuation file."),
                default={}),
    cfg.DictOpt('sriov_resource_driver_mappings',
                help=_("A mappping driver names for certain resource "
                       "names. Expected that device of VIF related to "
                       "exact physnet should be binded on specified driver."),
                default={}),
    cfg.StrOpt('zun_cni_config_file',
               help=_("Path to the Zun CNI config file."),
               default='/etc/cni/net.d/10-zun-cni.conf'),
]

ALL_OPTS = (daemon_opts)


def register_opts(conf):
    conf.register_group(cni_daemon_group)
    conf.register_opts(ALL_OPTS, cni_daemon_group)


def list_opts():
    return {cni_daemon_group: ALL_OPTS}



#    Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Zun common internal object model"""

from oslo_versionedobjects import base as ovoo_base
from oslo_versionedobjects import fields as ovoo_fields

from zun.objects import fields as obj_fields

remotable_classmethod = ovoo_base.remotable_classmethod
remotable = ovoo_base.remotable


class ZunObjectRegistry(ovoo_base.VersionedObjectRegistry):
    pass


class ZunObject(ovoo_base.VersionedObject):
    """Base class and object factory.

    This forms the base of all objects that can be remoted or instantiated
    via RPC. Simply defining a class that inherits from this base class
    will make it remotely instantiatable. Objects should implement the
    necessary "get" classmethod routines as well as "save" object methods
    as appropriate.
    """
    OBJ_SERIAL_NAMESPACE = 'zun_object'
    OBJ_PROJECT_NAMESPACE = 'zun'

    def as_dict(self):
        return {k: getattr(self, k)
                for k in self.fields
                if self.obj_attr_is_set(k)}


class ZunPersistentObject(object):
    """Mixin class for Persistent objects.

    This adds the fields that we use in common for all persistent objects.
    """
    fields = {
        'created_at': ovoo_fields.DateTimeField(nullable=True,
                                                tzinfo_aware=False),
        'updated_at': ovoo_fields.DateTimeField(nullable=True,
                                                tzinfo_aware=False),
    }


class ZunObjectSerializer(ovoo_base.VersionedObjectSerializer):
    # Base class to use for object hydration
    OBJ_BASE_CLASS = ZunObject


class ObjectListBase(ovoo_base.ObjectListBase):
    # NOTE: These are for transition to using the oslo
    # base object and can be removed when we move to it.
    @classmethod
    def _obj_primitive_key(cls, field):
        return 'zun_object.%s' % field

    @classmethod
    def _obj_primitive_field(cls, primitive, field,
                             default=obj_fields.UnspecifiedDefault):
        key = cls._obj_primitive_key(field)
        if default == obj_fields.UnspecifiedDefault:
            return primitive[key]
        else:
            return primitive.get(key, default)


def obj_to_primitive(obj):
    """Recursively turn an object into a python primitive.

    A ZunObject becomes a dict, and anything that implements ObjectListBase
    becomes a list.
    """
    if isinstance(obj, ObjectListBase):
        return [obj_to_primitive(x) for x in obj]
    elif isinstance(obj, ZunObject):
        result = {}
        for key in obj.obj_fields:
            if obj.obj_attr_is_set(key) or key in obj.obj_extra_fields:
                result[key] = obj_to_primitive(getattr(obj, key))
        return result
    else:
        return obj


def obj_equal_prims(obj_1, obj_2, ignore=None):
    """Compare two primitives for equivalence ignoring some keys.

    This operation tests the primitives of two objects for equivalence.
    Object primitives may contain a list identifying fields that have been
    changed - this is ignored in the comparison. The ignore parameter lists
    any other keys to be ignored.

    :param:obj1: The first object in the comparison
    :param:obj2: The second object in the comparison
    :param:ignore: A list of fields to ignore
    :returns: True if the primitives are equal ignoring changes
    and specified fields, otherwise False.
    """

    def _strip(prim, keys):
        if isinstance(prim, dict):
            for k in keys:
                prim.pop(k, None)
            for v in prim.values():
                _strip(v, keys)
        if isinstance(prim, list):
            for v in prim:
                _strip(v, keys)
        return prim

    if ignore is not None:
        keys = ['zun_object.changes'] + ignore
    else:
        keys = ['zun_object.changes']
    prim_1 = _strip(obj_1.obj_to_primitive(), keys)
    prim_2 = _strip(obj_2.obj_to_primitive(), keys)
    return prim_1 == prim_2



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import retrying

from os_vif import objects as obj_vif
from oslo_concurrency import lockutils
from oslo_config import cfg
from oslo_log import log as logging

from zun.cni.binding import base as b_base
from zun.cni import utils as cni_utils
from zun.common import consts
from zun.common import context as zun_context
from zun.common import exception
from zun.network import neutron
from zun import objects


LOG = logging.getLogger(__name__)
CONF = cfg.CONF
RETRY_DELAY = 1000  # 1 second in milliseconds
TYPE_CAPSULE = 'CAPSULE'
TYPE_CONTAINER = 'CONTAINER'


class ZunCNIRegistryPlugin(object):
    def __init__(self, registry):
        self.registry = registry
        self.host = CONF.host
        self.context = zun_context.get_admin_context(all_projects=True)
        self.neutron_api = neutron.NeutronAPI(self.context)

    def _get_container_uuid(self, params):
        # NOTE(hongbin): The runtime should set K8S_POD_NAME as
        # capsule/container uuid
        return params.args.K8S_POD_NAME

    def _get_container_type(self, params):
        return getattr(params.args, 'ZUN_CONTAINER_TYPE', TYPE_CAPSULE)

    def add(self, params):
        vifs = self._do_work(params, b_base.connect)

        container_uuid = self._get_container_uuid(params)

        # NOTE(dulek): Saving containerid to be able to distinguish old DEL
        #              requests that we should ignore. We need a lock to
        #              prevent race conditions and replace whole object in the
        #              dict for multiprocessing.Manager to notice that.
        with lockutils.lock(container_uuid, external=True):
            self.registry[container_uuid] = {
                'containerid': params.CNI_CONTAINERID,
                'vif_unplugged': False,
                'del_received': False,
                'vifs': {ifname: {'active': vif.active, 'id': vif.id}
                         for ifname, vif in vifs.items()},
            }
            LOG.debug('Saved containerid = %s for capsule/container %s',
                      params.CNI_CONTAINERID, container_uuid)

        # Wait for VIFs to become active.
        timeout = CONF.cni_daemon.vif_active_timeout

        def any_vif_inactive(vifs):
            """Return True if there is at least one VIF that's not ACTIVE."""
            return any(not vif['active'] for vif in vifs.values())

        # Wait for timeout sec, 1 sec between tries, retry when even one
        # vif is not active.
        @retrying.retry(stop_max_delay=timeout * 1000, wait_fixed=RETRY_DELAY,
                        retry_on_result=any_vif_inactive)
        def wait_for_active(container_uuid):
            return self.registry[container_uuid]['vifs']

        result = wait_for_active(container_uuid)
        for vif in result.values():
            if not vif['active']:
                LOG.error("Timed out waiting for vifs to become active")
                raise exception.ResourceNotReady(resource=container_uuid)

        return vifs[consts.DEFAULT_IFNAME]

    def delete(self, params):
        container_uuid = self._get_container_uuid(params)
        try:
            reg_ci = self.registry[container_uuid]['containerid']
            LOG.debug('Read containerid = %s for capsule/container %s',
                      reg_ci, container_uuid)
            if reg_ci and reg_ci != params.CNI_CONTAINERID:
                # NOTE(dulek): This is a DEL request for some older (probably
                #              failed) ADD call. We should ignore it or we'll
                #              unplug a running capsule/container.
                LOG.warning('Received DEL request for unknown ADD call. '
                            'Ignoring.')
                return
        except KeyError:
            pass

        try:
            self._do_work(params, b_base.disconnect)
        except exception.ContainerNotFound:
            LOG.warning('Capsule/Container is not found in DB. Ignoring.')
            pass

        # NOTE(ndesh): We need to lock here to avoid race condition
        #              with the deletion code in the watcher to ensure that
        #              we delete the registry entry exactly once
        try:
            with lockutils.lock(container_uuid, external=True):
                if self.registry[container_uuid]['del_received']:
                    LOG.debug("Remove capsule/container %(container)s from "
                              "registry", {'container': container_uuid})
                    del self.registry[container_uuid]
                else:
                    LOG.debug("unplug vif for capsule/container %(container)s",
                              {'container': container_uuid})
                    container_dict = self.registry[container_uuid]
                    container_dict['vif_unplugged'] = True
                    self.registry[container_uuid] = container_dict
        except KeyError:
            # This means the capsule/container was removed before vif was
            # unplugged. This shouldn't happen, but we can't do anything
            # about it now
            LOG.debug('Capsule/Container %s not found while handling DEL '
                      'request. Ignoring.', container_uuid)
            pass

    def _do_work(self, params, fn):
        container_uuid = self._get_container_uuid(params)
        container_type = self._get_container_type(params)

        if container_type == TYPE_CAPSULE:
            container = objects.Capsule.get_by_uuid(self.context,
                                                    container_uuid)
        elif container_type == TYPE_CONTAINER:
            container = objects.Container.get_by_uuid(self.context,
                                                      container_uuid)
        else:
            raise exception.CNIError('Unexpected type: %s' % container_type)

        vifs = cni_utils.get_vifs(container)

        for ifname, vif in vifs.items():
            is_default_gateway = (ifname == consts.DEFAULT_IFNAME)
            if is_default_gateway:
                # NOTE(ygupta): if this is the default interface, we should
                # use the ifname supplied in the CNI ADD request
                ifname = params.CNI_IFNAME

            fn(vif, self._get_inst(container), ifname, params.CNI_NETNS,
               is_default_gateway=is_default_gateway,
               container_id=params.CNI_CONTAINERID)
        return vifs

    def _get_inst(self, container):
        return obj_vif.instance_info.InstanceInfo(
            uuid=container.uuid, name=container.name)



#    Copyright 2014 Red Hat Inc.
#    Copyright 2017 IBM Corp
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_versionedobjects import fields

from zun.common import exception
from zun.objects import base


@base.ZunObjectRegistry.register
class NUMANode(base.ZunObject):
    # Version 1.0: Initial version
    VERSION = '1.0'

    fields = {
        'id': fields.IntegerField(read_only=True),
        'cpuset': fields.SetOfIntegersField(),
        'pinned_cpus': fields.SetOfIntegersField(),
        'mem_total': fields.IntegerField(nullable=True),
        'mem_available': fields.IntegerField(nullable=True),
        }

    @property
    def free_cpus(self):
        return self.cpuset - self.pinned_cpus or set()

    @property
    def avail_cpus(self):
        return len(self.free_cpus)

    def pin_cpus(self, cpus):
        if cpus - self.cpuset:
            raise exception.CPUPinningUnknown(requested=list(cpus),
                                              cpuset=list(self.pinned_cpus))
        if self.pinned_cpus & cpus:
            raise exception.CPUPinningInvalid(requested=list(cpus),
                                              free=list(self.cpuset -
                                                        self.pinned_cpus))
        self.pinned_cpus |= cpus

    def unpin_cpus(self, cpus):
        if cpus - self.cpuset:
            raise exception.CPUUnpinningUnknown(requested=list(cpus),
                                                cpuset=list(self.pinned_cpus))
        if (self.pinned_cpus & cpus) != cpus:
            raise exception.CPUUnpinningInvalid(requested=list(cpus),
                                                pinned=list(self.pinned_cpus))
        self.pinned_cpus -= cpus

    def _to_dict(self):
        return {
            'id': self.id,
            'cpuset': list(self.cpuset),
            'pinned_cpus': list(self.pinned_cpus),
            'mem_total': self.mem_total,
            'mem_available': self.mem_available
            }

    @classmethod
    def _from_dict(cls, data_dict):
        cpuset = set(data_dict.get('cpuset', ''))
        node_id = data_dict.get('id')
        pinned_cpus = set(data_dict.get('pinned_cpus'))
        mem_total = data_dict.get('mem_total')
        mem_available = data_dict.get('mem_available')
        return cls(id=node_id, cpuset=cpuset,
                   pinned_cpus=pinned_cpus,
                   mem_total=mem_total,
                   mem_available=mem_available)


@base.ZunObjectRegistry.register
class NUMATopology(base.ZunObject):
    # Version 1.0: Initial version
    VERSION = '1.0'

    fields = {
        'nodes': fields.ListOfObjectsField('NUMANode'),
        }

    @classmethod
    def _from_dict(cls, data_dict):
        return cls(nodes=[
            NUMANode._from_dict(node_dict)
            for node_dict in data_dict.get('nodes', [])])

    def _to_dict(self):
        return {
            'nodes': [n._to_dict() for n in self.nodes],
        }

    def to_list(self):
        return [n._to_dict() for n in self.nodes]



# Copyright (c) 2017 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from zun.scheduler import filters

LOG = logging.getLogger(__name__)


class CPUFilter(filters.BaseHostFilter):
    """DEPRECATED: Filter the containers by cpu request"""

    run_filter_once_per_request = True

    def __init__(self):
        super(CPUFilter, self).__init__()
        LOG.warning('The CPUFilter is deprecated since the 7.0.0 '
                    'Wallaby release. VCPU filtering is performed natively '
                    'using the Placement service when using the '
                    'filter_scheduler driver. Operators should define cpu '
                    'allocation ratios either per host in the zun.conf '
                    'or via the placement API.')

    def host_passes(self, host_state, container, extra_spec):
        if not container.cpu:
            return True

        cpu_free = host_state.cpus - host_state.cpu_used
        if cpu_free < container.cpu:
            LOG.debug("%(host_state)s does not have %(container_vcpus).2f "
                      "usable vcpus, it only has %(free_vcpus).2f usable "
                      "vcpus",
                      {'host_state': host_state,
                       'container_vcpus': container.cpu,
                       'free_vcpus': cpu_free})
            return False
        host_state.limits['cpu'] = host_state.cpus
        return True



# Copyright 2015 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg


sql_opts = [
    cfg.StrOpt('mysql_engine',
               default='InnoDB',
               help='MySQL engine to use.')
]

DEFAULT_OPTS = (sql_opts)


def register_opts(conf):
    conf.register_opts(sql_opts, 'database')


def list_opts():
    return {"DEFAULT": DEFAULT_OPTS}



# Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""add availability_zone to service

Revision ID: 3f49fa520409
Revises: 50829990c965
Create Date: 2018-02-10 22:33:22.890723

"""

# revision identifiers, used by Alembic.
revision = '3f49fa520409'
down_revision = '50829990c965'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('zun_service',
                  sa.Column('availability_zone', sa.String(255),
                            nullable=True))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_utils import timeutils

import zun.conf
from zun import objects

CONF = zun.conf.CONF


class ServiceGroup(object):
    def __init__(self):
        self.service_down_time = CONF.service_down_time

    def service_is_up(self, member):
        if not isinstance(member, objects.ZunService):
            raise TypeError
        if member.forced_down:
            return False

        last_heartbeat = (member.last_seen_up or
                          member.updated_at or member.created_at)
        now = timeutils.utcnow()
        elapsed = timeutils.delta_seconds(last_heartbeat, now)
        is_up = abs(elapsed) <= self.service_down_time
        return is_up



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
import pecan

from zun.api.controllers import base
from zun.api.controllers.v1 import collection
from zun.api.controllers.v1.schemas import network as schema
from zun.api.controllers.v1.views import network_view as view
from zun.api import utils as api_utils
from zun.api import validation
from zun.common import exception
from zun.common import policy
from zun import objects


LOG = logging.getLogger(__name__)


def _get_network(context, network_ident):
    networks = objects.ZunNetwork.list(
        context,
        filters={'neutron_net_id': network_ident})
    if not networks:
        raise exception.NetworkNotFound(network=network_ident)

    return networks[0]


class NetworkCollection(collection.Collection):
    """API representation of a collection of network."""

    fields = {
        'network'
    }

    """A list containing network objects"""

    def __init__(self, **kwargs):
        super(NetworkCollection, self).__init__(**kwargs)
        self._type = 'network'

    @staticmethod
    def convert_with_links(rpc_network, limit, url=None,
                           expand=False, **kwargs):
        collection = NetworkCollection()
        collection.network = [view.format_network(url, p) for p in rpc_network]
        collection.next = collection.get_next(limit, url=url, **kwargs)
        return collection


class NetworkController(base.Controller):
    """Controller for Network"""

    @pecan.expose('json')
    @api_utils.enforce_content_types(['application/json'])
    @exception.wrap_pecan_controller_exception
    @validation.validated(schema.network_create)
    def post(self, **network_dict):
        """Create a new network.

        :param network_dict: a network within the request body.
        """
        context = pecan.request.context
        policy.enforce(context, "network:create", action="network:create")
        new_network = pecan.request.compute_api.network_create(
            context, network_dict['neutron_net_id'])
        return view.format_network(pecan.request.host_url, new_network)

    @base.Controller.api_version("1.27")  # noqa
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def delete(self, network_ident, **kwargs):
        """Delete a network.

        :param network_ident: UUID of the network.
        """
        context = pecan.request.context
        policy.enforce(context, "network:delete", action="network:delete")
        context.all_projects = True
        network = _get_network(context, network_ident)
        compute_api = pecan.request.compute_api
        compute_api.network_delete(context, network)
        pecan.response.status = 204



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Manages information about the host.
"""

from oslo_log import log as logging

from zun.common import consts
from zun.common import exception
from zun.common import utils
import zun.conf
from zun.container.docker import utils as docker_utils

CONF = zun.conf.CONF
LOG = logging.getLogger(__name__)


class Host(object):

    def __init__(self):
        self._hostname = None

    def get_hostname(self):
        """Returns the hostname of the host."""
        with docker_utils.docker_client() as docker:
            hostname = docker.info()['Name']
            if self._hostname is None:
                self._hostname = hostname
            elif hostname != self._hostname:
                self._hostname = hostname
                LOG.warning('Hostname has changed from %(old)s '
                            'to %(new)s. A restart is required '
                            'to take effect.',
                            {'old': self._hostname, 'new': hostname})
        return self._hostname

    def get_storage_info(self):
        with docker_utils.docker_client() as docker:
            info = docker.info()
            storage_driver = str(info['Driver'])
            # DriverStatus is list. Convert it to dict
            driver_status = dict(info['DriverStatus'])
            backing_filesystem = \
                str(driver_status.get('Backing Filesystem'))
            default_base_size = driver_status.get('Base Device Size')
            if default_base_size:
                default_base_size = float(default_base_size.strip('GB'))
        return {
            'storage_driver': storage_driver,
            'backing_filesystem': backing_filesystem,
            'default_base_size': default_base_size
        }

    def check_supported_disk_quota(self, host_info):
        """Check your system be supported disk quota or not"""
        storage_info = self.get_storage_info()
        sp_disk_quota = True
        storage_driver = storage_info['storage_driver']
        backing_filesystem = storage_info['backing_filesystem']
        if storage_driver not in consts.SUPPORTED_STORAGE_DRIVERS:
            sp_disk_quota = False
        else:
            if storage_driver == 'overlay2':
                if backing_filesystem == 'xfs':
                    # Check project quota mount option
                    try:
                        cmd = ("mount |grep $(df " +
                               host_info['docker_root_dir'] +
                               " |awk 'FNR==2 {print $1}') | grep 'xfs'"
                               " |grep -E 'pquota|prjquota'")
                        utils.execute(cmd, shell=True)
                    except exception.CommandError:
                        sp_disk_quota = False
                else:
                    sp_disk_quota = False
        return sp_disk_quota



# Copyright 2018 Arm Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""change properties of restart policy in capsule

Revision ID: 6ff4d35f4334
Revises: d9714eadbdc2
Create Date: 2018-01-26 17:19:59.564188

"""

# revision identifiers, used by Alembic.
revision = '6ff4d35f4334'
down_revision = 'd9714eadbdc2'
branch_labels = None
depends_on = None

from alembic import op

import sqlalchemy as sa


def upgrade():
    op.alter_column('capsule', 'restart_policy',
                    type_=sa.String(length=255)
                    )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from zun.api.controllers.v1.schemas import parameter_types


_registry_properties = {
    'name': parameter_types.registry_name,
    'domain': parameter_types.registry_domain,
    'username': parameter_types.registry_username,
    'password': parameter_types.registry_password,
}

registry_create = {
    'type': 'object',
    'properties': {
        'registry': {
            'type': 'object',
            'properties': _registry_properties,
            'additionalProperties': False,
            'required': ['domain'],
        },
    },
    'required': ['registry'],
    'additionalProperties': False,
}

_registry_update_properties = {
    'name': parameter_types.registry_name,
    'domain': parameter_types.registry_domain,
    'username': parameter_types.registry_username,
    'password': parameter_types.registry_password,
}

registry_update = {
    'type': 'object',
    'properties': {
        'registry': {
            'type': 'object',
            'properties': _registry_update_properties,
            'additionalProperties': False,
        },
    },
    'required': ['registry'],
    'additionalProperties': False,
}



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from zun.api import utils
from zun.common import exception
from zun.tests import base


class TestUtils(base.BaseTestCase):
    """Test cases for zun.api.utils"""

    def test_validate_limit(self):
        self.assertEqual(1000, utils.validate_limit(None))
        self.assertEqual(1000, utils.validate_limit(1001))
        self.assertEqual(50, utils.validate_limit(50))
        with self.assertRaisesRegex(exception.InvalidValue,
                                    "Limit must be positive"):
            utils.validate_limit(-1)
        with self.assertRaisesRegex(exception.InvalidValue,
                                    "Limit must be positive"):
            utils.validate_limit(0)
        with self.assertRaisesRegex(exception.InvalidValue,
                                    "Limit must be positive integer"):
            utils.validate_limit('a')
        with self.assertRaisesRegex(exception.InvalidValue,
                                    "Limit must be positive integer"):
            utils.validate_limit('5.5')

    def test_validate_sort_dir(self):
        self.assertEqual('asc', utils.validate_sort_dir('asc'))
        self.assertEqual('desc', utils.validate_sort_dir('desc'))
        with self.assertRaisesRegex(exception.InvalidValue,
                                    "Invalid sort direction"):
            utils.validate_sort_dir(None)
        with self.assertRaisesRegex(exception.InvalidValue,
                                    "Invalid sort direction"):
            utils.validate_sort_dir('abc')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import functools

from zun.api.validation import validators


def validated(request_body_schema):
    """Register a schema to validate a resource reference.

    Registered schema will be used for validating a request body just before
    API method execution.

    :param request_body_schema: a schema to validate the resource reference
    """
    schema_validator = validators.SchemaValidator(request_body_schema,
                                                  is_body=True)

    def add_validator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            schema_validator.validate(kwargs)
            return func(*args, **kwargs)
        return wrapper
    return add_validator


def validate_query_param(req, query_param_schema):
    """Register a schema to validate a resource reference.

    Registered schema will be used for validating a request query params
    just before API method execution.

    :param req: the request object
    :param query_param_schema: a schema to validate the resource reference
    """

    schema_validator = validators.SchemaValidator(query_param_schema,
                                                  is_body=False)

    def add_validator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            schema_validator.validate(req.params.mixed())
            return func(*args, **kwargs)
        return wrapper
    return add_validator



# Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from unittest import mock
from unittest.mock import patch

from oslo_utils import uuidutils
from webtest.app import AppError

from zun.common import exception
import zun.conf
from zun import objects
from zun.tests.unit.api import base as api_base
from zun.tests.unit.db import utils


CONF = zun.conf.CONF


class TestImageController(api_base.FunctionalTest):
    @mock.patch('zun.common.policy.enforce', return_value=True)
    @patch('zun.compute.api.API.image_pull')
    @patch('zun.objects.ComputeNode.get_by_name')
    def test_image_pull(self, mock_get, mock_image_pull, mock_policy_enforce):
        mock_image_pull.side_effect = lambda x, y: y
        mock_get.return_value = mock.Mock(hostname='fake-host')

        params = ('{"repo": "hello-world", "host": "fake-host"}')
        response = self.post('/v1/images/',
                             params=params,
                             content_type='application/json')

        self.assertEqual(202, response.status_int)
        self.assertTrue(mock_image_pull.called)

        params = ('{"repo": "hello-world:test", "host": "fake-host"}')
        response = self.post('/v1/images/',
                             params=params,
                             content_type='application/json')

        self.assertEqual(202, response.status_int)
        self.assertTrue(mock_image_pull.called)

    @patch('zun.compute.api.API.image_pull')
    def test_image_pull_with_no_repo(self, mock_image_pull):
        params = {}
        with self.assertRaisesRegex(AppError,
                                    "is a required property"):
            self.post('/v1/images/',
                      params=params,
                      content_type='application/json')
        self.assertTrue(mock_image_pull.not_called)

    @mock.patch('zun.common.policy.enforce', return_value=True)
    @patch('zun.compute.api.API.image_pull')
    @patch('zun.objects.ComputeNode.get_by_name')
    def test_image_pull_conflict(
            self, mock_get, mock_image_pull, mock_policy_enforce):
        mock_image_pull.side_effect = lambda x, y: y
        mock_get.return_value = mock.Mock(hostname='fake-host')

        params = ('{"repo": "hello-world", "host": "fake-host"}')
        response = self.post('/v1/images/',
                             params=params,
                             content_type='application/json')

        self.assertEqual(202, response.status_int)
        self.assertTrue(mock_image_pull.called)
        self.assertRaises(AppError, self.post, '/v1/images/',
                          params=params, content_type='application/json')
        self.assertTrue(mock_image_pull.not_called)

    @mock.patch('zun.common.policy.enforce', return_value=True)
    @patch('zun.compute.api.API.image_pull')
    @patch('zun.objects.ComputeNode.get_by_name')
    def test_pull_image_set_project_id_and_user_id(
            self, mock_get, mock_image_pull, mock_policy_enforce):
        def _create_side_effect(cnxt, image):
            self.assertEqual(self.context.project_id, image.project_id)
            self.assertEqual(self.context.user_id, image.user_id)
            return image
        mock_image_pull.side_effect = _create_side_effect
        mock_get.return_value = mock.Mock(hostname='fake-host')

        params = ('{"repo": "hello-world", "host": "foo"}')
        self.post('/v1/images/',
                  params=params,
                  content_type='application/json')

    @mock.patch('zun.common.policy.enforce', return_value=True)
    @patch('zun.compute.api.API.image_pull')
    @patch('zun.objects.ComputeNode.get_by_name')
    def test_image_pull_with_tag(
            self, mock_get, mock_image_pull, mock_policy_enforce):
        mock_image_pull.side_effect = lambda x, y: y
        mock_get.return_value = mock.Mock(hostname='fake-host')

        params = ('{"repo": "hello-world:latest", "host": "fake-host"}')
        response = self.post('/v1/images/',
                             params=params,
                             content_type='application/json')

        self.assertEqual(202, response.status_int)
        self.assertTrue(mock_image_pull.called)

    @mock.patch('zun.common.policy.enforce', return_value=True)
    @patch('zun.objects.Image.list')
    def test_get_all_images(self, mock_image_list, mock_policy_enforce):
        test_image = utils.get_test_image()
        images = [objects.Image(self.context, **test_image)]
        mock_image_list.return_value = images

        response = self.get('/v1/images/')

        mock_image_list.assert_called_once_with(mock.ANY,
                                                1000, None, 'id', 'asc',
                                                filters=None)
        self.assertEqual(200, response.status_int)
        actual_images = response.json['images']
        self.assertEqual(1, len(actual_images))
        self.assertEqual(test_image['uuid'],
                         actual_images[0].get('uuid'))
        self.assertEqual(test_image['host'],
                         actual_images[0].get('host'))

    @patch('zun.common.policy.enforce')
    @patch('zun.objects.Image.get_by_uuid')
    def test_get_one_image_by_uuid(self, mock_image_get_by_uuid, mock_policy):
        mock_policy.return_value = True
        test_image = utils.get_test_image()
        test_image_obj = objects.Image(self.context, **test_image)
        mock_image_get_by_uuid.return_value = test_image_obj

        response = self.get('/v1/images/%s/' % test_image['uuid'])
        mock_image_get_by_uuid.assert_called_once_with(
            mock.ANY,
            test_image['uuid'])
        self.assertEqual(200, response.status_int)
        self.assertEqual(test_image['uuid'],
                         response.json['uuid'])
        self.assertEqual(test_image['host'],
                         response.json['host'])

    @mock.patch('zun.common.policy.enforce', return_value=True)
    @patch('zun.objects.Image.list')
    def test_get_all_images_with_pagination_marker(
            self, mock_image_list, mock_policy_enforce):
        image_list = []
        for id_ in range(4):
            test_image = utils.create_test_image(
                context=self.context,
                id=id_,
                repo='testrepo' + str(id_),
                uuid=uuidutils.generate_uuid())
            image_list.append(objects.Image(self.context, **test_image))
        mock_image_list.return_value = image_list[-1:]
        response = self.get('/v1/images/?limit=3&marker=%s'
                            % image_list[2].uuid)

        self.assertEqual(200, response.status_int)
        actual_images = response.json['images']
        self.assertEqual(1, len(actual_images))
        self.assertEqual(image_list[-1].uuid,
                         actual_images[0].get('uuid'))

    @patch('zun.compute.api.API.image_search')
    def test_search_image(self, mock_image_search):
        mock_image_search.return_value = {'name': 'redis', 'stars': 2000}
        response = self.get('/v1/images/redis/search/')
        self.assertEqual(200, response.status_int)
        mock_image_search.assert_called_once_with(
            mock.ANY, 'redis', CONF.default_image_driver, False, None)

    @patch('zun.compute.api.API.image_search')
    def test_search_image_with_tag(self, mock_image_search):
        mock_image_search.return_value = {'name': 'redis', 'stars': 2000}
        response = self.get('/v1/images/redis:test/search/')
        self.assertEqual(200, response.status_int)
        mock_image_search.assert_called_once_with(
            mock.ANY, 'redis:test', CONF.default_image_driver, False, None)

    @patch('zun.compute.api.API.image_search')
    def test_search_image_not_found(self, mock_image_search):
        mock_image_search.side_effect = exception.ImageNotFound
        self.assertRaises(AppError, self.get, '/v1/images/redis/search/')
        mock_image_search.assert_called_once_with(
            mock.ANY, 'redis', CONF.default_image_driver, False, None)

    @patch('zun.compute.rpcapi.API.image_search')
    def test_search_image_with_exact_match_true(self, mock_image_search):
        mock_image_search.return_value = {'name': 'redis', 'stars': 2000}
        response = self.get(
            '/v1/images/redis/search?exact_match=true&image_driver=docker')
        self.assertEqual(200, response.status_int)
        mock_image_search.assert_called_once_with(
            mock.ANY, 'redis', 'docker', True, None)

    @patch('zun.compute.rpcapi.API.image_search')
    def test_search_image_with_exact_match_false(self, mock_image_search):
        mock_image_search.return_value = {'name': 'redis', 'stars': 2000}
        response = self.get(
            '/v1/images/redis/search?exact_match=false&image_driver=glance')
        self.assertEqual(200, response.status_int)
        mock_image_search.assert_called_once_with(
            mock.ANY, 'redis', 'glance', False, None)

    @patch('zun.compute.api.API.image_search')
    def test_search_image_with_exact_match_wrong(self, mock_image_search):
        mock_image_search.side_effect = exception.InvalidValue
        with self.assertRaisesRegex(AppError,
                                    "Invalid input for query parameters"):
            self.get('/v1/images/redis/search?exact_match=wrong')

    @patch('zun.compute.api.API.image_search')
    def test_search_image_with_image_driver_wrong(self, mock_image_search):
        mock_image_search.side_effect = exception.InvalidValue
        with self.assertRaisesRegex(AppError,
                                    "Invalid input for query parameters"):
            self.get('/v1/images/redis/search?image_driver=wrong')


class TestImageEnforcement(api_base.FunctionalTest):

    def _common_policy_check(self, rule, func, *arg, **kwarg):
        self.policy.set_rules({rule: 'project_id:non_fake'})
        response = func(*arg, **kwarg)
        self.assertEqual(403, response.status_int)
        self.assertEqual('application/json', response.content_type)
        self.assertTrue(
            "Policy doesn't allow %s to be performed." % rule,
            response.json['errors'][0]['detail'])

    def test_policy_disallow_get_all(self):
        self._common_policy_check(
            'image:get_all', self.get_json, '/images/',
            expect_errors=True)

    def test_policy_disallow_create(self):
        params = ('{"repo": "foo", "host": "foo"}')
        self._common_policy_check(
            'image:pull', self.post, '/v1/images/',
            params=params,
            content_type='application/json',
            expect_errors=True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Add container number info to compute node

Revision ID: 4bf34495d060
Revises: 53a8b515057e
Create Date: 2017-04-17 10:06:17.340957

"""

# revision identifiers, used by Alembic.
revision = '4bf34495d060'
down_revision = '53a8b515057e'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('compute_node',
                  sa.Column('total_containers', sa.Integer(), nullable=False))
    op.add_column('compute_node',
                  sa.Column('running_containers',
                            sa.Integer(), nullable=False))
    op.add_column('compute_node',
                  sa.Column('paused_containers', sa.Integer(), nullable=False))
    op.add_column('compute_node',
                  sa.Column('stopped_containers',
                            sa.Integer(), nullable=False))



# Copyright (c) 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Policy Engine For zun."""

from oslo_log import log as logging
from oslo_policy import opts
from oslo_policy import policy
from oslo_utils import excutils

from zun.common import exception
from zun.common import policies
import zun.conf

_ENFORCER = None
CONF = zun.conf.CONF
LOG = logging.getLogger(__name__)

# TODO(gmann): Remove setting the default value of config policy_file
# once oslo_policy change the default value to 'policy.yaml'.
# https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
DEFAULT_POLICY_FILE = 'policy.yaml'
opts.set_defaults(CONF, DEFAULT_POLICY_FILE)


# we can get a policy enforcer by this init.
# oslo policy support change policy rule dynamically.
# at present, policy.enforce will reload the policy rules when it checks
# the policy files have been touched.
def init(policy_file=None, rules=None,
         default_rule=None, use_conf=True, overwrite=True):
    """Init an Enforcer class.

        :param policy_file: Custom policy file to use, if none is
                            specified, ``conf.policy_file`` will be
                            used.
        :param rules: Default dictionary / Rules to use. It will be
                      considered just in the first instantiation. If
                      :meth:`load_rules` with ``force_reload=True``,
                      :meth:`clear` or :meth:`set_rules` with
                      ``overwrite=True`` is called this will be overwritten.
        :param default_rule: Default rule to use, conf.default_rule will
                             be used if none is specified.
        :param use_conf: Whether to load rules from cache or config file.
        :param overwrite: Whether to overwrite existing rules when reload rules
                          from config file.
    """
    global _ENFORCER
    if not _ENFORCER:
        # https://docs.openstack.org/oslo.policy/latest/user/usage.html
        _ENFORCER = policy.Enforcer(CONF,
                                    policy_file=policy_file,
                                    rules=rules,
                                    default_rule=default_rule,
                                    use_conf=use_conf,
                                    overwrite=overwrite)
        register_rules(_ENFORCER)
    return _ENFORCER


def register_rules(enforcer):
    enforcer.register_defaults(policies.list_rules())


def enforce(context, rule=None, target=None,
            do_raise=True, exc=None, *args, **kwargs):

    """Checks authorization of a rule against the target and credentials.

        :param dict context: As much information about the user performing the
                             action as possible.
        :param rule: The rule to evaluate.
        :param dict target: As much information about the object being operated
                            on as possible.
        :param do_raise: Whether to raise an exception or not if check
                         fails.
        :param exc: Class of the exception to raise if the check fails.
                    Any remaining arguments passed to :meth:`enforce` (both
                    positional and keyword arguments) will be passed to
                    the exception class. If not specified,
                    :class:`PolicyNotAuthorized` will be used.

        :return: ``False`` if the policy does not allow the action and `exc` is
                 not provided; otherwise, returns a value that evaluates to
                 ``True``.  Note: for rules using the "case" expression, this
                 ``True`` value will be the specified string from the
                 expression.
    """
    enforcer = init()
    credentials = context.to_policy_values()
    if not exc:
        exc = exception.PolicyNotAuthorized
    if target is None:
        target = {'project_id': context.project_id,
                  'user_id': context.user_id}
    return enforcer.enforce(rule, target, credentials,
                            do_raise=do_raise, exc=exc, *args, **kwargs)


def authorize(context, action, target, do_raise=True, exc=None,
              might_not_exist=False):
    """Verifies that the action is valid on the target in this context.

       :param context: zun context
       :param action: string representing the action to be checked
           this should be colon separated for clarity.
           i.e. ``network:attach_external_network``
       :param target: dictionary representing the object of the action
           for object creation this should be a dictionary representing the
           location of the object e.g. ``{'project_id': context.project_id}``
       :param do_raise: if True (the default), raises PolicyNotAuthorized;
           if False, returns False
       :param exc: Class of the exception to raise if the check fails.
            Any remaining arguments passed to :meth:`authorize` (both
            positional and keyword arguments) will be passed to
            the exception class. If not specified,
            :class:`PolicyNotAuthorized` will be used.
       :param might_not_exist: If True the policy check is skipped (and the
            function returns True) if the specified policy does not exist.
            Defaults to false.

       :raises zun.common.exception.PolicyNotAuthorized: if verification fails
           and do_raise is True. Or if 'exc' is specified it will raise an
           exception of that type.

       :return: returns a non-False value (not necessarily "True") if
           authorized, and the exact value False if not authorized and
           do_raise is False.
    """
    credentials = context.to_policy_values()
    if not exc:
        exc = exception.PolicyNotAuthorized
    if might_not_exist and not (_ENFORCER.rules and action in _ENFORCER.rules):
        return True
    try:
        result = _ENFORCER.enforce(action, target, credentials,
                                   do_raise=do_raise, exc=exc, action=action)
    except Exception:
        with excutils.save_and_reraise_exception():
            LOG.debug('Policy check for %(action)s failed with credentials '
                      '%(credentials)s',
                      {'action': action, 'credentials': credentials})
    return result


def check_is_admin(context):
    """Whether or not user is admin according to policy setting.

    """
    init()
    target = {}
    credentials = context.to_policy_values()
    return _ENFORCER.enforce('context_is_admin', target, credentials)



# Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""add image_pull_policy column

Revision ID: 43e1088c3389
Revises: c5565cbaa3de
Create Date: 2016-11-17 09:26:22.756296

"""

# revision identifiers, used by Alembic.
revision = '43e1088c3389'
down_revision = 'c5565cbaa3de'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('container',
                  sa.Column('image_pull_policy', sa.Text(),
                            nullable=True))



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_utils import uuidutils
from testtools.matchers import HasLength

from zun import objects
from zun.tests.unit.db import base
from zun.tests.unit.db import utils


class TestVolumeMappingObject(base.DbTestCase):

    def setUp(self):
        super(TestVolumeMappingObject, self).setUp()
        self.fake_volume_mapping = utils.get_test_volume_mapping()
        self.fake_volume = utils.get_test_volume()

    def test_get_by_uuid(self):
        uuid = self.fake_volume_mapping['uuid']
        with mock.patch.object(self.dbapi, 'get_volume_mapping_by_uuid',
                               autospec=True) as mock_get_volume_mapping:
            mock_get_volume_mapping.return_value = self.fake_volume_mapping
            volume_mapping = objects.VolumeMapping.get_by_uuid(self.context,
                                                               uuid)
            mock_get_volume_mapping.assert_called_once_with(self.context, uuid)
            self.assertEqual(self.context, volume_mapping._context)

    def test_list(self):
        with mock.patch.object(self.dbapi, 'list_volume_mappings',
                               autospec=True) as mock_get_list:
            mock_get_list.return_value = [self.fake_volume_mapping]
            volume_mappings = objects.VolumeMapping.list(self.context)
            self.assertEqual(1, mock_get_list.call_count)
            self.assertThat(volume_mappings, HasLength(1))
            self.assertIsInstance(volume_mappings[0], objects.VolumeMapping)
            self.assertEqual(self.context, volume_mappings[0]._context)

    def test_list_by_container(self):
        with mock.patch.object(self.dbapi, 'list_volume_mappings',
                               autospec=True) as mock_get_list:
            mock_get_list.return_value = [self.fake_volume_mapping]
            volume_mappings = objects.VolumeMapping.list_by_container(
                self.context, 'fake_container_uuid')
            mock_get_list.assert_called_once_with(
                self.context, {'container_uuid': 'fake_container_uuid'},
                None, None, None, None)
            self.assertThat(volume_mappings, HasLength(1))
            self.assertIsInstance(volume_mappings[0], objects.VolumeMapping)
            self.assertEqual(self.context, volume_mappings[0]._context)

    def test_list_with_filters(self):
        with mock.patch.object(self.dbapi, 'list_volume_mappings',
                               autospec=True) as mock_get_list:
            mock_get_list.return_value = [self.fake_volume_mapping]
            filt = {'volume_provider': 'fake_provider'}
            volume_mappings = objects.VolumeMapping.list(self.context,
                                                         filters=filt)
            self.assertEqual(1, mock_get_list.call_count)
            self.assertThat(volume_mappings, HasLength(1))
            self.assertIsInstance(volume_mappings[0], objects.VolumeMapping)
            self.assertEqual(self.context, volume_mappings[0]._context)
            mock_get_list.assert_called_once_with(self.context,
                                                  filters=filt,
                                                  limit=None, marker=None,
                                                  sort_key=None, sort_dir=None)

    def test_create(self):
        with mock.patch.object(
            self.dbapi, 'create_volume_mapping',
            autospec=True
        ) as mock_create_volume_mapping, mock.patch.object(
            self.dbapi, 'create_volume',
            autospec=True
        ) as mock_create_volume:
            mock_create_volume_mapping.return_value = self.fake_volume_mapping
            mock_create_volume.return_value = self.fake_volume
            volume_mapping_dict = dict(self.fake_volume_mapping)
            volume_dict = dict(self.fake_volume)
            for attr in ('id', 'uuid', 'created_at', 'updated_at'):
                volume_mapping_dict.pop(attr)
                volume_dict.pop(attr)
            volume_mapping = dict(volume_dict)
            volume_mapping.update(volume_mapping_dict)
            volume_mapping.pop('volume_id')
            volume_mapping = objects.VolumeMapping(self.context,
                                                   **volume_mapping)
            volume_mapping.create(self.context)
            mock_create_volume_mapping.assert_called_once_with(
                self.context, volume_mapping_dict)
            mock_create_volume.assert_called_once_with(
                self.context, volume_dict)
            self.assertEqual(self.context, volume_mapping._context)

    def test_destroy(self):
        uuid = self.fake_volume_mapping['uuid']
        volume_id = self.fake_volume['id']
        with mock.patch.object(
            self.dbapi, 'get_volume_mapping_by_uuid',
            autospec=True
        ) as mock_get_volume_mapping, mock.patch.object(
            self.dbapi, 'destroy_volume_mapping',
            autospec=True
        ) as mock_destroy_volume_mapping, mock.patch.object(
            self.dbapi, 'destroy_volume',
            autospec=True
        ) as mock_destroy_volume:
            mock_get_volume_mapping.return_value = self.fake_volume_mapping
            volume_mapping = objects.VolumeMapping.get_by_uuid(
                self.context, uuid)
            volume_mapping.destroy(self.context)
            mock_get_volume_mapping.assert_called_once_with(
                self.context, uuid)
            mock_destroy_volume_mapping.assert_called_once_with(
                self.context, uuid)
            mock_destroy_volume.assert_called_once_with(
                self.context, volume_id)
            self.assertEqual(self.context, volume_mapping._context)

    def test_save(self):
        uuid = self.fake_volume_mapping['uuid']
        volume_id = self.fake_volume['id']
        volume_uuid = self.fake_volume['uuid']
        with mock.patch.object(
            self.dbapi, 'get_volume_mapping_by_uuid',
            autospec=True
        ) as mock_get_volume_mapping, mock.patch.object(
            self.dbapi, 'update_volume_mapping',
            autospec=True
        ) as mock_update_volume_mapping, mock.patch.object(
            self.dbapi, 'get_volume_by_id',
            autospec=True
        ) as mock_get_volume, mock.patch.object(
            self.dbapi, 'update_volume',
            autospec=True
        ) as mock_update_volume:
            mock_get_volume_mapping.return_value = self.fake_volume_mapping
            mock_get_volume.return_value = self.fake_volume
            volume_mapping = objects.VolumeMapping.get_by_uuid(
                self.context, uuid)
            new_container_uuid = uuidutils.generate_uuid()
            volume_mapping.container_uuid = new_container_uuid
            volume_mapping.connection_info = 'new_info'
            volume_mapping.save()

            mock_get_volume_mapping.assert_called_once_with(
                self.context, uuid)
            mock_get_volume.assert_called_once_with(
                self.context, volume_id)
            mock_update_volume_mapping.assert_called_once_with(
                None, uuid, {'container_uuid': new_container_uuid})
            mock_update_volume.assert_called_once_with(
                None, volume_uuid, {'connection_info': 'new_info'})
            self.assertEqual(self.context, volume_mapping._context)

    def test_refresh(self):
        uuid = self.fake_volume_mapping['uuid']
        new_uuid = uuidutils.generate_uuid()
        returns = [dict(self.fake_volume_mapping, uuid=uuid),
                   dict(self.fake_volume_mapping, uuid=new_uuid)]
        expected = [mock.call(self.context, uuid),
                    mock.call(self.context, uuid)]
        with mock.patch.object(self.dbapi, 'get_volume_mapping_by_uuid',
                               side_effect=returns,
                               autospec=True) as mock_get_volume_mapping:
            volume_mapping = objects.VolumeMapping.get_by_uuid(self.context,
                                                               uuid)
            self.assertEqual(uuid, volume_mapping.uuid)
            volume_mapping.refresh()
            self.assertEqual(new_uuid, volume_mapping.uuid)
            self.assertEqual(expected, mock_get_volume_mapping.call_args_list)
            self.assertEqual(self.context, volume_mapping._context)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Version 1 of the Zun API

NOTE: IN PROGRESS AND NOT FULLY IMPLEMENTED.
"""

from oslo_log import log as logging
import pecan

from zun.api.controllers import base as controllers_base
from zun.api.controllers import link
from zun.api.controllers.v1 import availability_zone as a_zone
from zun.api.controllers.v1 import capsules as capsule_controller
from zun.api.controllers.v1 import containers as container_controller
from zun.api.controllers.v1 import hosts as host_controller
from zun.api.controllers.v1 import images as image_controller
from zun.api.controllers.v1 import networks as network_controller
from zun.api.controllers.v1 import quota_classes as quota_classes_controller
from zun.api.controllers.v1 import quotas as quotas_controller
from zun.api.controllers.v1 import registries as registries_controller
from zun.api.controllers.v1 import zun_services
from zun.api.controllers import versions as ver
from zun.api import http_error
from zun.common.i18n import _

LOG = logging.getLogger(__name__)


BASE_VERSION = 1

MIN_VER_STR = '%s %s' % (ver.Version.service_string, ver.BASE_VER)

MAX_VER_STR = '%s %s' % (ver.Version.service_string, ver.CURRENT_MAX_VER)

MIN_VER = ver.Version({ver.Version.string: MIN_VER_STR},
                      MIN_VER_STR, MAX_VER_STR)
MAX_VER = ver.Version({ver.Version.string: MAX_VER_STR},
                      MIN_VER_STR, MAX_VER_STR)


class MediaType(controllers_base.APIBase):
    """A media type representation."""

    fields = (
        'base',
        'type',
    )


class V1(controllers_base.APIBase):
    """The representation of the version 1 of the API."""

    fields = (
        'id',
        'media_types',
        'links',
        'services',
        'containers',
        'images',
        'networks',
        'hosts',
        'capsules',
        'availability_zones',
        'quotas',
        'quota_classes',
        'registries',
    )

    @staticmethod
    def convert():
        v1 = V1()
        v1.id = "v1"
        v1.links = [link.make_link('self', pecan.request.host_url,
                                   'v1', '', bookmark=True),
                    link.make_link('describedby',
                                   'https://docs.openstack.org',
                                   'developer/zun/dev',
                                   'api-spec-v1.html',
                                   bookmark=True, type='text/html')]
        v1.media_types = [MediaType(
            base='application/json',
            type='application/vnd.openstack.zun.v1+json')]
        v1.services = [link.make_link('self', pecan.request.host_url,
                                      'services', ''),
                       link.make_link('bookmark',
                                      pecan.request.host_url,
                                      'services', '',
                                      bookmark=True)]
        v1.containers = [link.make_link('self', pecan.request.host_url,
                                        'containers', ''),
                         link.make_link('bookmark',
                                        pecan.request.host_url,
                                        'containers', '',
                                        bookmark=True)]
        v1.images = [link.make_link('self', pecan.request.host_url,
                                    'images', ''),
                     link.make_link('bookmark',
                                    pecan.request.host_url,
                                    'images', '',
                                    bookmark=True)]
        v1.networks = [link.make_link('self', pecan.request.host_url,
                                      'networks', ''),
                       link.make_link('bookmark',
                                      pecan.request.host_url,
                                      'networks', '',
                                      bookmark=True)]
        v1.hosts = [link.make_link('self', pecan.request.host_url,
                                   'hosts', ''),
                    link.make_link('bookmark',
                                   pecan.request.host_url,
                                   'hosts', '',
                                   bookmark=True)]
        v1.availability_zones = [link.make_link('self', pecan.request.host_url,
                                                'availability_zones', ''),
                                 link.make_link('bookmark',
                                                pecan.request.host_url,
                                                'availability_zones', '',
                                                bookmark=True)]
        v1.capsules = [link.make_link('self', pecan.request.host_url,
                                      'capsules', ''),
                       link.make_link('bookmark',
                                      pecan.request.host_url,
                                      'capsules', '',
                                      bookmark=True)]
        v1.quotas = [link.make_link('self', pecan.request.host_url,
                                    'quotas', ''),
                     link.make_link('bookmark',
                                    pecan.request.host_url,
                                    'quotas', '',
                                    bookmark=True)]
        v1.quota_classes = [link.make_link('self', pecan.request.host_url,
                                           'quota_classes', ''),
                            link.make_link('bookmark',
                                           pecan.request.host_url,
                                           'quota_classes', '',
                                           bookmark=True)]
        v1.registries = [link.make_link('self', pecan.request.host_url,
                                        'registries', ''),
                         link.make_link('bookmark', pecan.request.host_url,
                                        'registries', '', bookmark=True)]
        return v1


class Controller(controllers_base.Controller):
    """Version 1 API controller root."""

    services = zun_services.ZunServiceController()
    containers = container_controller.ContainersController()
    images = image_controller.ImagesController()
    networks = network_controller.NetworkController()
    hosts = host_controller.HostController()
    availability_zones = a_zone.AvailabilityZoneController()
    capsules = capsule_controller.CapsuleController()
    quotas = quotas_controller.QuotaController()
    quota_classes = quota_classes_controller.QuotaClassController()
    registries = registries_controller.RegistryController()

    @pecan.expose('json')
    def get(self):
        return V1.convert()

    def _check_version(self, version, headers=None):
        if headers is None:
            headers = {}
        # ensure that major version in the URL matches the header
        if version.major != BASE_VERSION:
            raise http_error.HTTPNotAcceptableAPIVersion(_(
                "Mutually exclusive versions requested. Version %(ver)s "
                "requested but not supported by this service. "
                "The supported version range is: "
                "[%(min)s, %(max)s].") % {'ver': version,
                                          'min': MIN_VER_STR,
                                          'max': MAX_VER_STR},
                headers=headers,
                max_version=str(MAX_VER),
                min_version=str(MIN_VER))
        # ensure the minor version is within the supported range
        if version < MIN_VER or version > MAX_VER:
            raise http_error.HTTPNotAcceptableAPIVersion(_(
                "Version %(ver)s was requested but the minor version is not "
                "supported by this service. The supported version range is: "
                "[%(min)s, %(max)s].") % {'ver': version, 'min': MIN_VER_STR,
                                          'max': MAX_VER_STR},
                headers=headers,
                max_version=str(MAX_VER),
                min_version=str(MIN_VER))

    @pecan.expose()
    def _route(self, args):
        version = ver.Version(
            pecan.request.headers, MIN_VER_STR, MAX_VER_STR)

        # Always set the basic version headers
        pecan.response.headers[ver.Version.min_string] = MIN_VER_STR
        pecan.response.headers[ver.Version.max_string] = MAX_VER_STR
        pecan.response.headers[ver.Version.string] = " ".join(
            [ver.Version.service_string, str(version)])
        pecan.response.headers["vary"] = ver.Version.string

        # assert that requested version is supported
        self._check_version(version, pecan.response.headers)
        pecan.request.version = version
        if pecan.request.body:
            msg = ("Processing request: url: %(url)s, %(method)s, "
                   "body: %(body)s" %
                   {'url': pecan.request.url,
                    'method': pecan.request.method,
                    'body': pecan.request.body})
            LOG.debug(msg)

        return super(Controller, self)._route(args)


__all__ = ('Controller',)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

""" Implements linux net utils"""

from oslo_log import log as logging

from zun.common import utils


LOG = logging.getLogger(__name__)


def _ovs_vsctl(args, timeout=None):
    full_args = ['ovs-vsctl']
    if timeout is not None:
        full_args += ['--timeout=%s' % timeout]
    full_args += args
    try:
        return utils.execute(*full_args, run_as_root=True)
    except Exception as e:
        LOG.error("Unable to execute %(cmd)s. Exception: %(exception)s",
                  {'cmd': full_args, 'exception': e})
        raise


def _create_ovs_vif_cmd(bridge, dev, iface_id, mac, instance_id):
    cmd = ['--', '--if-exists', 'del-port', dev, '--',
           'add-port', bridge, dev,
           '--', 'set', 'Interface', dev,
           'external-ids:iface-id=%s' % iface_id,
           'external-ids:iface-status=active',
           'external-ids:attached-mac=%s' % mac,
           'external-ids:vm-uuid=%s' % instance_id]
    return cmd


def create_ovs_vif_port(bridge, dev, iface_id, mac, instance_id):
    # TODO(hongbin): switch to ovsdb
    _ovs_vsctl(_create_ovs_vif_cmd(bridge, dev, iface_id, mac, instance_id))


def delete_ovs_vif_port(bridge, dev):
    # TODO(hongbin): switch to ovsdb
    _ovs_vsctl(['--', '--if-exists', 'del-port', bridge, dev])
